//File:         Cable.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              This class is used to connect components such as servers/clients
//              to switches or routers.
//

package NetLayout;

import java.io.*;
import java.awt.*;
import java.util.*; // for StringTokenizer and Vector
import java.lang.*;


public class Cable extends Link {
  static int objNo = 0;
  
  public Cable(Net net, Node n1, Node n2, int x, int y) {
    this.type = Net.CABLE;
    ID = net.objID++;
    net.addComponentDelay(ID);
    this.net = net;
    this.name = "Cable" + net.objNo[this.type]++;
    this.n1 = n1;
    this.n2 = n2;
    timeForwardChannel = 0.0;
    timeBackwardChannel = 0.0;
    n1.addLink(this);
    n2.addLink(this);
    if (n2.type == Net.MALINK) {
      this.x = x;
      this.y = y;

    } else {
      this.x = n2.x+16;
      this.y = n2.y+16;
    }
    if (n2.type == Net.MALINK) {
      MALink l = (MALink) n2;
      l.addNode(n1);
    }
    if (n2.type == Net.ROUTER) {
      Router r = (Router) n2;
      r.addNode(n1);
    }
    if (n2.type == Net.SWITCH) {
      Switch s = (Switch) n2;
      s.addNode(n1);
    }
    if (n1.type == Net.MALINK) {
      MALink l = (MALink) n1;
      l.addNode(n2);
    }
    if (n1.type == Net.ROUTER) {
      Router r = (Router) n1;
      r.addNode(n2);
    }
    if (n1.type == Net.SWITCH) {
      Switch s = (Switch) n1;
      s.addNode(n2);
    }
    net.links.addElement(this);
  }
  public Cable(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    av = new AttrValue(st);
    try {
      Double r = Double.valueOf(av.value);
      distance = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      bandwidth = r.doubleValue();
      // System.out.println(r);
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    av = new AttrValue(st);
    n1 = net.searchNode(Integer.parseInt(av.value));
    n1.addLink(this);
    av = new AttrValue(st);
    n2 = net.searchNode(Integer.parseInt(av.value));
    n2.addLink(this);
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    net.links.addElement(this);
    if (n2.type == Net.MALINK) {
      MALink l = (MALink) n2;
      l.addNode(n1);
    }
    if (n2.type == Net.ROUTER) {
      Router r = (Router) n2;
      r.addNode(n1);
    }
    if (n2.type == Net.SWITCH) {
      Switch s = (Switch) n2;
      s.addNode(n1);
    }
    if (n1.type == Net.MALINK) {
      MALink l = (MALink) n1;
      l.addNode(n2);
    }
    if (n1.type == Net.ROUTER) {
      Router r = (Router) n1;
      r.addNode(n2);
    }
    if (n1.type == Net.SWITCH) {
      Switch s = (Switch) n1;
      s.addNode(n2);
    }
    timeForwardChannel = 0.0;
    timeBackwardChannel = 0.0;
    net.objNo[type]++;
    //System.out.println(toString());
  }

  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " distance="+distance+" bandwidth="+bandwidth+
      " n1ID="+n1.ID+" n2ID="+n2.ID+" x="+x+" y="+y+"\n";
  }
    
}




