//File:         CacheServer.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Model the cache server
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector

public class CacheServer extends Node {
  Stat reqDocSt;
  protected double cacheSize;    // limit the # and size of cached documents

  public CacheServer(Net net, int x, int y) {
    this.x = x;
    this.y = y;
    this.net = net;
    ID = net.objID++;
    type = net.CACHE;
    cacheSize = 1.0E+9;
    name =  "cache" + net.objNo[type]++;
    reqDocSt = new Stat();
    reqDocSt.avg = 10000; reqDocSt.min = 2000; reqDocSt.max = 1000000;
    net.nodes.addElement(this);
  }
  public CacheServer(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.objID++;
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);

    try {
      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      cacheSize = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      power = r.doubleValue();
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    //System.out.println(toString());
    reqDocSt = new Stat();
    reqDocSt.avg = 40000; reqDocSt.min = 40000; reqDocSt.max = 40000;
    localTime = 0.0;
    net.nodes.addElement(this);
    net.objNo[type]++;
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+" cacheSize="+cacheSize+
      " lon="+lon+" lat="+lat+" power="+power+
      linkListToString()+
      "\n";
  }
  public void eventHandle(EventRecord er) {
    double queueingTime = localTime - er.time;
    if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
    }
    totalQueueingTime += queueingTime;
    noOfRequests++;

    String msgType = er.getField("msgType");
    if (msgType.compareTo("http-request") == 0) {

     try {
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double msgProcessingTime=(net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime;

      // retrieve document
      int docIndex = Integer.parseInt(er.getField("docIndex"));
      double docSize = ((0.0+docIndex)/net.noOfDocs )*(reqDocSt.max-reqDocSt.min)+reqDocSt.min;

      // calculate the retrieval time
      double retrieveTime = docSize/(power*net.retrieveSpeed);
      // we may consider random service time for file retrieval
      localTime += retrieveTime;
      double msgProcessingTime2=(net.msgProcessingTime+docSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime2;

      // Find return link
      Link l = net.getNextLink(ID, er.src);
      if (l == null) {
        Node n = net.searchNode(er.dst);
        if (n != null)
          net.println("This server "+name+" is not connected to the "+n.name);
        else
          net.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;

      double transmissionTime = docSize/l.bandwidth;

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, docSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, docSize, localTime);
      }

      er.time = time;
      er.next = nextNode.ID;
      er.dst = er.src;
      er.src = ID;
      er.addField("msgType", "http-response");
      er.addField("msgSize", ""+docSize);
      er.addField("retrieveTime", ""+retrieveTime);

      d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();
      accQueueingTime += queueingTime;
      er.addField("queueingTime", ""+accQueueingTime);

      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += msgProcessingTime+msgProcessingTime2;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();
      accTransmissionTime += transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);

      d = Double.valueOf(er.getField("propagationTime"));
      double accPropagationTime = d.doubleValue();



      accPropagationTime += propagationTime;
      er.addField("propagationTime", ""+accPropagationTime);

      String cds = er.getField("componentDelay");
      String serverCDS = " "+ID+"-QT="+queueingTime+",MPT="+(msgProcessingTime+msgProcessingTime2)+",RT="+retrieveTime;
      String linkCDS = " "+l.ID+"-PT="+propagationTime+",TT="+transmissionTime;
      er.addField("componentDelay", cds+serverCDS+linkCDS);


      // generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = l.n2.x+16; p2y = l.n2.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }
      net.el.queueEvent(er);
      net.println("Cache Server ID="+ID+"time="+localTime+" name="+name+
          " return reqID="+er.getField("reqID")+" msgSize="+docSize+
          " queueingTime="+queueingTime);
     } catch (NumberFormatException e) {
      System.err.println("CacheServer: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
     }
    }
  }
}

