//File:         ComponentDelay.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              class for parsing component delay string
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector
import java.lang.*;
import java.text.*;

public class ComponentDelay {
  public int ID;
  public double queueingTime;
  public double msgProcessingTime;
  public double propagationTime;
  public double transmissionTime;
  public double retrieveTime;
  public double totalDelay;
  public double percentage;
  public int rank;
  public ComponentDelay(int id) {
    ID=id; queueingTime=0.0; msgProcessingTime=0.0; propagationTime=0.0;
    transmissionTime=0.0;  retrieveTime=0.0; totalDelay=0.0; percentage = 0.0;
    rank = 0;
  }
  public ComponentDelay(String SID, String cdList) {
    ID=0; queueingTime=0.0; msgProcessingTime=0.0; propagationTime=0.0;
    transmissionTime=0.0;  retrieveTime=0.0; percentage = 0.0;
    rank = 0;
    try {
      ID = Integer.parseInt(SID);
      StringTokenizer st = new StringTokenizer(cdList, ",");
      while (st.hasMoreTokens()) {
        StringTokenizer steq=new StringTokenizer(st.nextToken(), "=");
        String delayName = steq.nextToken();
        String delayValue = steq.nextToken();
        Double d = Double.valueOf(delayValue);
        double delay  = d.doubleValue();
        if (delayName.compareTo("QT")==0) queueingTime = delay;
        if (delayName.compareTo("MPT")==0) msgProcessingTime = delay;
        if (delayName.compareTo("PT")==0) propagationTime = delay;
        if (delayName.compareTo("TT")==0) transmissionTime = delay;
        if (delayName.compareTo("RT")==0) retrieveTime = delay;
      }
      computeTotalDelay();
    }  catch (NumberFormatException  e) {
        System.err.println("numberFormatException");
    }  catch (NoSuchElementException e) {
        System.err.println("no more tokens");
    }
  }
  public String toString() {
    return "ID="+ID+" QT="+queueingTime+" MPT="+msgProcessingTime+
           " PT="+propagationTime+" TT="+transmissionTime+
           " RT="+retrieveTime+" TD="+totalDelay+
           " %="+percentage+" rank="+rank;
  }
  public void computeTotalDelay() {
    totalDelay = transmissionTime+propagationTime+msgProcessingTime+
                   queueingTime+retrieveTime;
  }
}
