//File:         ComponentDelayList.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector
import java.lang.*;
import java.text.*;

public class ComponentDelayList {
  String sessionID;
  Vector v;
  Net net;
  double totalDelay;
  public ComponentDelayList(Net n, String session, String cds) {
    sessionID = session;
    v = new Vector();
    net = n;

    String IDString="";
    String cdList="";
   try {
    StringTokenizer st = new StringTokenizer(cds);
    while (st.hasMoreTokens()) {
      StringTokenizer stid = new StringTokenizer(st.nextToken(), ":");
      IDString = stid.nextToken();
      cdList = stid.nextToken();
      ComponentDelay cd = new ComponentDelay(IDString, cdList);
      if (notMergeList(cd)) v.addElement(cd);
    }
   } catch (NoSuchElementException e) {
     System.err.println("session="+session+"cds="+cds+"IDString="+IDString+
        "cdList="+cdList+" e="+e.toString());
   }
    totalDelay = 0.0;
    for (int i=0; i<v.size(); i++) {
      ComponentDelay cd = (ComponentDelay) v.elementAt(i);
      totalDelay += cd.totalDelay;
    }
    sort();
    setPercentage();
    //displayComponentDelayVector();
  }
  public ComponentDelayList(Net n) {
    net = n;
    v = new Vector();
  }
  public void addComponentDelay(int id) {
    ComponentDelay cd = new ComponentDelay(id);
    v.addElement(cd);
  }
  public boolean notMergeList(ComponentDelay cd) {
    for (int i=0; i<v.size(); i++) {
      ComponentDelay cdi = (ComponentDelay) v.elementAt(i);
      if (cdi.ID == cd.ID) {
        // merge the delays
        cdi.queueingTime +=cd.queueingTime;
        cdi.msgProcessingTime += cd.msgProcessingTime;
        cdi.propagationTime += cd.propagationTime;
        cdi.transmissionTime += cd.transmissionTime;
        // need to recompute totalDelay
        cdi.computeTotalDelay();
        return false;
      }
    }
    return true;
  }
  public void sort() {
    //quickSort();
    bubbleSort();
  }
  public void bubbleSort() {
    ComponentDelay temp;
    boolean exchange = true;
    for (int i=0; exchange && i<v.size(); i++) {
       exchange = false;
       for (int j=0; j<v.size()-i-1; j++) {
          ComponentDelay cd1 = (ComponentDelay) v.elementAt(j);
          ComponentDelay cd2 = (ComponentDelay) v.elementAt(j+1);
          if (cd1.totalDelay>cd2.totalDelay) {
            exchange = true;
            v.setElementAt(cd1, j+1);
            v.setElementAt(cd2, j);
          }
       }
    }
  }
  public void quickSortDelay(int lo0, int hi0){

    double mid;

    int lo = lo0;
    int hi = hi0;
 

    if ( hi0 > lo0 ){

       // mid = p[ (lo0 + hi0) / 2 ].age;
       ComponentDelay cd = (ComponentDelay) v.elementAt((lo0 + hi0) / 2);
       mid = cd.totalDelay;

       while( lo <= hi ){

    while ( ( lo < hi0 ) && ( ((ComponentDelay) v.elementAt(lo)).totalDelay < mid ))++lo;

    while ( ( hi > lo0 ) && ( ((ComponentDelay) v.elementAt(hi)).totalDelay > mid ))--hi;

        if( lo <= hi ){
           swap(lo, hi);
           ++lo;
           --hi;
        }

     }

     if( lo0 < hi)
        quickSortDelay(lo0, hi);

     if( lo < hi0)
        quickSortDelay(lo, hi0);
     }
   }

   private void swap(int i, int j)
   {
      Object tempi, tempj;
      tempi = v.elementAt(i);
      tempj = v.elementAt(j);
      v.setElementAt(tempi,j);
      v.setElementAt(tempj,i);
   }

   public void quickSort() {
      quickSortDelay(0,  v.size() - 1);
   }
   public void displayComponentDelayVector() {
     System.out.println("Component Delay Analysis of Session "+sessionID);
     for (int i=0; i< v.size(); i++) {
       ComponentDelay cd = (ComponentDelay) v.elementAt(i);
       System.out.println(cd.toString());
     }
   }
   public String toString() {
     String s="Component Delay Analysis:\n";
     for (int i=0; i< v.size(); i++) {
       ComponentDelay cd = (ComponentDelay) v.elementAt(i);
       String name = net.getNameWithID(cd.ID);
       s += "Name="+name+" " + cd.toString()+"\n";
     }
     return s;
   }
   public void setPercentage() {
     for (int i=0; i< v.size(); i++) {
       ComponentDelay cd = (ComponentDelay) v.elementAt(i);
       cd.percentage = cd.totalDelay/totalDelay;
       cd.rank = i;
     }
   }
   public static void main(String[] args) {
     String cd1="1:QT=10,MPT=100 3:PT=5,TT=100 5:QT=20,MPT=50 6:PT=10,TT=50 7:QT=20,MPT=200,RT=40";
     String cd2=" 6:PT=10,TT=50 5:QT=50,MPT=50 3:PT=5,TT=100 1:QT=30,MPT=100";
     Net net = new Net();
     // after adding net, the simple main() test will not work.
     // since there is no name saved for the fake ID data above
     ComponentDelayList cdl = new ComponentDelayList(net, "SS1", cd1+cd2);
   }
}
