//File:         DNSProperty.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//

package NetLayout;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class DNSProperty extends Dialog implements ActionListener {
  final static int size = 30;
  Node n;
  DNS cl;
  NetLayoutFrame f;

  JPanel panel1 = new JPanel();

  JPanel panel3 = new JPanel();
  JPanel insetsPanel1 = new JPanel();

  JPanel insetsPanel3 = new JPanel();
  JSeparator sep1 = new JSeparator();
  JSeparator sep2 = new JSeparator();
  JSeparator sep3 = new JSeparator();
  Label statusBar = new Label();

  JButton button1 = new JButton();
  JButton button2 = new JButton();
  JLabel label1 = new JLabel();
  JLabel label2 = new JLabel();
  Label nameLabel = new Label("Name:", Label.RIGHT);
  TextField nameField;
  Label idLabel = new Label("ID:", Label.RIGHT);
  Label idField; // change not allowed
  Label xLabel = new Label("X:", Label.RIGHT);
  TextField xField;
  Label yLabel = new Label("Y:", Label.RIGHT);
  TextField yField;
  /* we will need root dns servers field here if we model DNS operation
  Label dnsLabel = new Label("DNS Server Name:", Label.RIGHT);
  TextField dnsField;
  */

  Label lonLabel = new Label("Longitude:", Label.RIGHT);
  TextField lonField;
  Label latLabel = new Label("Latitude:", Label.RIGHT);
  TextField latField;
  Label powerLabel = new Label("Power:", Label.RIGHT);
  TextField powerField;

  Label linkListLabel = new Label("Connected to:", Label.RIGHT);
  Label linkListField;

  BorderLayout borderLayout1 = new BorderLayout();

  BorderLayout borderLayout3 = new BorderLayout();
  FlowLayout flowLayout1 = new FlowLayout();

  String product = "Network Layout Tool Copyright (c) 1998";
  String dialogName = "DNS Property";

  public DNSProperty(Frame parent, DNS c) {
    super(parent);
    f = (NetLayoutFrame) parent;

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try  {
      jbInit(c);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    pack();
  }

  private void jbInit(DNS c) throws Exception  {
    this.setTitle("DNS Property");
    cl = c;
    setResizable(false);
    panel1.setLayout(borderLayout1);

    panel3.setLayout(borderLayout3);
    insetsPanel1.setLayout(flowLayout1);

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    insetsPanel3.setLayout(gbl);

    label1.setText(product);
    label2.setText(dialogName);

    button1.setText("UPDATE");
    button1.addActionListener(this);
    button2.setText("CANCEL");
    button2.addActionListener(this);
    insetsPanel1.add(button1, null);
    insetsPanel1.add(button2, null);
    panel1.add(insetsPanel1, BorderLayout.SOUTH);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(label1, gbc);
    insetsPanel3.add(label1);
    
    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(label2, gbc);
    insetsPanel3.add(label2);

    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep1, gbc);
    insetsPanel3.add(sep1);
    gbc.fill = 0;


    // add code to create name value pair for each property item

    nameField = new TextField(c.name, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(nameLabel, gbc);
    insetsPanel3.add(nameLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(nameField, gbc);
    insetsPanel3.add(nameField);
    
    idField = new Label(Integer.toString(c.ID),Label.LEFT);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(idLabel, gbc);
    insetsPanel3.add(idLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(idField, gbc);
    insetsPanel3.add(idField);

    xField = new TextField(Integer.toString(c.x), size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(xLabel, gbc);
    insetsPanel3.add(xLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(xField, gbc);
    insetsPanel3.add(xField);

    yField = new TextField(Integer.toString(c.y), size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(yLabel, gbc);
    insetsPanel3.add(yLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(yField, gbc);
    insetsPanel3.add(yField);

    /*    To model root DNS server

    n = c.net.searchNode(c.dnsID);
    dnsField = new TextField(n.name, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(dnsLabel, gbc);
    insetsPanel3.add(dnsLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(dnsField, gbc);
    insetsPanel3.add(dnsField);
    */


    lonField = new TextField(""+c.lon, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(lonLabel, gbc);
    insetsPanel3.add(lonLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(lonField, gbc);
    insetsPanel3.add(lonField);

    latField = new TextField(""+c.lat, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(latLabel, gbc);
    insetsPanel3.add(latLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(latField, gbc);
    insetsPanel3.add(latField);

    powerField = new TextField(""+c.power, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(powerLabel, gbc);
    insetsPanel3.add(powerLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(powerField, gbc);
    insetsPanel3.add(powerField);


    linkListField = new Label(c.linkListNameToString(), Label.LEFT);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(linkListLabel, gbc);
    insetsPanel3.add(linkListLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(linkListField, gbc);
    insetsPanel3.add(linkListField);

    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep2, gbc);
    insetsPanel3.add(sep2);

    gbc.anchor = GridBagConstraints.SOUTH;
    Label statusBarLabel = new Label("status:");
    gbc.gridwidth = 1;
    gbl.setConstraints(statusBarLabel, gbc);
    insetsPanel3.add(statusBarLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(statusBar, gbc);
    insetsPanel3.add(statusBar);

    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep3, gbc);
    insetsPanel3.add(sep3);

    panel3.add(insetsPanel3,BorderLayout.CENTER);
    panel1.add(panel3, BorderLayout.CENTER);
    this.add(panel1, null);
  }

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  void cancel() {
    dispose();
  }
  boolean update() {
    String ns = nameField.getText();
    cl.name = ns;
    ns = xField.getText();
    cl.x = Integer.parseInt(ns);
    ns = yField.getText();
    cl.y = Integer.parseInt(ns);

    /* model root DNS server
    ns = dnsField.getText();
    int id = cl.net.getID(ns);
    if (id == Net.NOTFOUND) {
      statusBar.setText("The DNS="+ns+" not exist. Check again.");
      return false;
    }
    cl.dnsID = id;
    */


    try {
      ns = lonField.getText();
      Double r = Double.valueOf(ns);
      cl.lon = r.doubleValue();
      ns = latField.getText();
      r = Double.valueOf(ns);
      cl.lat = r.doubleValue();
      ns = powerField.getText();
      r = Double.valueOf(ns);
      cl.power = r.doubleValue();
      
    } catch (NumberFormatException e) {
      statusBar.setText("Could not convert string to double/int. ");
      return false;
    }
    f.mapArea.redo();
    return true;
    //System.out.println("update DNS property, get new name="+newName);
  }
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == button1) {
      if (update()) cancel();
    }
    if (e.getSource() == button2) {
      cancel();
    }
  }
}

