//File:         LBA.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Load Balancing Agent class
//              Behavior not implemented yet
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector

public class LBA extends Node {           
  public LBA(Net net, int x, int y) {
    this.x = x;
    this.y = y;
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    type = net.LBA;
    name =  "LBA" + net.objNo[type]++;
    net.nodes.addElement(this);
  }
  public LBA(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    net.objID++;
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    try {
      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      power = r.doubleValue();
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    localTime = 0.0;
    net.nodes.addElement(this);
    net.objNo[type]++;
    //System.out.println(toString());
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+
      " lon="+lon+" lat="+lat+" power="+power+
      linkListToString()+
      "\n";
  }
    public void eventHandle(EventRecord er) {
    //
    //   not implemented yet
    //
    //String msgType = er.getField("msgType");
    // don't care about msg type '
    // just forward it


      if (localTime < er.time) localTime = er.time;
      localTime += net.msgProcessingTime/power;
      // should consider different msgSize for LBA query later

      // Find next node on net.pred[][]
      Link l = net.getNextLink(ID, er.dst);

      if (l == null) {
        Node n = net.searchNode(er.dst);
        if (n != null)
          System.out.println("This LBA"+name+" is not connected to the "+n.name);
        else
          System.out.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;
      

      // calculate the transmission time
      double msgSize = Integer.parseInt(er.getField("msgSize"));
      double transmissionTime = msgSize/l.bandwidth;

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, localTime);
      }
      er.time = time;
      er.next = nextNode.ID;
      er.src = ID;
      // er.dst no change
      net.el.queueEvent(er);
  }

}

