//File:         Link.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//              3/9/2000   C. Edward Chow  add code report bandwidth within an time
//                                         window
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Basic Link class, parent class of cable, PTPLink
//

package NetLayout;


public class Link {
  final static int LINK_TOL = 7; // min. distance to a link to be considered close
  int type;
  boolean bidirectional;
  Net net;
  Node n1;
  Node n2;
  int x, y;
  int ID;
  double distance;
  String name;
  double bandwidth;
  double timeForwardChannel;  // next time that n1->n2 channel is available
  double timeBackwardChannel; // next time that n2->n1 channel is available
                              // here we assume a full dupex channel
  boolean tracked;
  double bitsSentOnFChannelCurWindow;   // since the beginning of the measurement window
  double bitsSentOnFChannelNextWindow;

  double bitsSentOnBChannelCurWindow;   // since the beginning of the measurement window
  double bitsSentOnBChannelNextWindow;


  public Link() {
    distance = 10.0;
    bandwidth = 10000000.0; // 10 Mbps default
    bidirectional = true;      // default is bi-directional
    timeBackwardChannel = 0.0;
    timeForwardChannel = 0.0;
    tracked = false;
    bitsSentOnFChannelCurWindow = 0.0;
    bitsSentOnFChannelNextWindow = 0.0;
    bitsSentOnBChannelCurWindow = 0.0;
    bitsSentOnBChannelNextWindow = 0.0;

  }
  // may consider unify link and node type and implement eventhandling also
  // on link type objects. It may simplify the coding.

  public void setChannelAvailableTime(Node na, double time) {
    // set the time when the channel is available
    // na is the sending node
    if (!bidirectional || na == n1) {         // when it is a shared channel
       timeForwardChannel=time;   // only use forward channel variables
    } else {
       timeBackwardChannel = time;
    }
  }
  public double getChannelAvailableTime(Node na) {
    // get the time when the channel is available
    // na is the sending node
    if (!bidirectional || na == n1) {         // when it is a shared channel
       return timeForwardChannel;   // only use forward channel variables
    } else {
       return timeBackwardChannel;
    }
  }
  public void sendMsgToChannel(Node na, double msgSize, double time) {
    if (!bidirectional || na == n1) {
       if (time > net.nextWindowStartTime) {
          // msg go to next window
          bitsSentOnFChannelNextWindow += msgSize;
       } else {
          double timeLeftInWindow = net.nextWindowStartTime - time;
          double bwLeft = timeLeftInWindow*bandwidth;
          if (bwLeft >= msgSize) {
             // msg can be sent through within the current window
             bitsSentOnFChannelCurWindow += msgSize;
          } else {
             // some sent in current window
             bitsSentOnFChannelNextWindow += (msgSize - bwLeft);
             bitsSentOnFChannelCurWindow += bwLeft;
          }
       }
    } else {
       if (time > net.nextWindowStartTime) {
          // msg go to next window
          bitsSentOnBChannelNextWindow += msgSize;
       } else {
          double timeLeftInWindow = net.nextWindowStartTime - time;
          double bwLeft = timeLeftInWindow*bandwidth;
          if (bwLeft >= msgSize) {
             // msg can be sent through within the current window
             bitsSentOnBChannelCurWindow += msgSize;
          } else {
             // some sent in current window
             bitsSentOnBChannelNextWindow += (msgSize - bwLeft);
             bitsSentOnBChannelCurWindow += bwLeft;
          }
       }
    }
  }
  public void displayABWInLastWindow(double time) {
     double abw = getABWInLastWindow(n1, time);
     net.plotProbingResults.c1.updateABW(time, abw);
     abw = getABWInLastWindow(n2, time);
     net.plotProbingResults.c1.updateABW2(time, abw);
  }
  public void displayABWInLastWindow(Node na, double time) {
     double abw = getABWInLastWindow(na, time);
     net.plotProbingResults.c1.updateABW(time, abw);
  }
  public double getABWInLastWindow(Node na, double time) {
     if (!bidirectional || na == n1) {

          double bwUsed = bitsSentOnFChannelCurWindow/net.abwReportPeriod;
          double bitsSFCCW = bitsSentOnFChannelCurWindow;
          // reset variables on the channels for next period
          double bwInWindow = bandwidth*net.abwReportPeriod;
          // calculate next window period's accumulated traffic
          if (bitsSentOnFChannelNextWindow >= bwInWindow) {
             bitsSentOnFChannelCurWindow = bwInWindow;
             bitsSentOnFChannelNextWindow = bitsSentOnFChannelNextWindow - bwInWindow;
          } else {
             bitsSentOnFChannelCurWindow = bitsSentOnFChannelNextWindow;
             bitsSentOnFChannelNextWindow = 0.0;
          }
          double abw = bandwidth - bwUsed;
          net.println("time="+time+", timeForwardChannel="+timeForwardChannel+
                             ", bitSentOnFChannelCurWindow="+bitsSentOnFChannelCurWindow+
                             ", bitSentOnFChannelNextWindow="+bitsSentOnFChannelNextWindow+
                             ", bwUsed="+bwUsed+", ABW="+ abw+", bitsSFCCW="+bitsSFCCW);
          return abw;

    } else {

         double bwUsed = bitsSentOnBChannelCurWindow/net.abwReportPeriod;
          // reset variables on the channels for next period
          double bwInWindow = bandwidth*net.abwReportPeriod;
          if (bitsSentOnBChannelNextWindow >= bwInWindow) {
             bitsSentOnBChannelCurWindow = bwInWindow;
             bitsSentOnBChannelNextWindow = bitsSentOnBChannelNextWindow - bwInWindow;
          } else {
             bitsSentOnBChannelCurWindow = bitsSentOnBChannelNextWindow;
             bitsSentOnBChannelNextWindow = 0.0;
          }
          double abw = bandwidth - bwUsed;
          net.println("time="+time+", timeBackwardChannel="+timeBackwardChannel+
                             ", bitSentOnBChannelCurWindow="+bitsSentOnFChannelCurWindow+
                             ", bitSentOnBChannelNextWindow="+bitsSentOnFChannelNextWindow+
                             ", bwUsed="+bwUsed+", ABW="+ abw);
          return abw;
    }

  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+"\n";
  }
  public String getName() {
    return name;
  }
  public double getDistance() {
    return distance;
  }
  public void setDistance(double dist) {
    distance = dist;
  }
  public double getBandwidth() {
    return bandwidth;
  }
  public void setBandwidth(double bandw) {
    bandwidth = bandw;
  }
  public Net getNet() {
    return net;
  }
  public int getID(){
    return ID;
  }
  public boolean getTrackingStatus() {
    return tracked;
  }
  void draw() {
  }
  public boolean inLink(int x, int y) {
      int x1 = n1.getX()+16;
      int y1 = n1.getY()+16;
      int x2, y2;
      if (this.type == Net.PTPLINK) {
        x2 = n2.getX()+16;
        y2 = n2.getY()+16;
      } else {
        x2 = this.x;
        y2 = this.y;
      }
      
      if (close_to_vector(x1, y1, x2, y2, x, y, Link.LINK_TOL)) return true;
      else return false;
  }
  public boolean close_to_vector(int x1, int y1, int x2, int y2, int xp, int yp, int d)
  {
   int   xmin, ymin, xmax, ymax;
   float  x, y, slope, D2, dx, dy, dist2;

   dist2 = (float) d * d;
   if (x1 < x2) {
    xmin = x1 - d; xmax = x2 + d;
   } else {
    xmin = x2 - d; xmax = x1 + d;
   }
   if (xp < xmin || xmax < xp) return(false);

   if (y1 < y2)  {
     ymin = y1 - d; ymax = y2 + d;
   } else {
       ymin = y2 - d; ymax = y1 + d;
   }
   if (yp < ymin || ymax < yp) return(false);

   if (x2 == x1) {
       x = x1;
       y = yp;
   } else if (y1 == y2) {
       x = xp;
       y = y1;
   } else {
       slope = ((float) (x2 - x1)) / ((float) (y2 - y1));
       y = (slope * (xp - x1 + slope * y1) + yp) / (1 + slope * slope);
       x = ((float) x1) + slope * (y - y1);
       }
   dx = ((float) xp) - x;
   dy = ((float) yp) - y;
   D2 = dx * dx + dy * dy;
   return(D2 < dist2);
  }

}