//File:         MALink.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              MultiAccess Link class models old Ethernet Coax Cable
//

package NetLayout;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*; // for StringTokenizer and Vector
import java.lang.*;

public class MALink extends Node {
  protected int x, y, w, h;
  protected Vector nodeList = new Vector (2,2);
  protected double bandwidth;

  
  public MALink(Net net, int x, int y, int w, int h) {
    this.type = net.MALINK;
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    bandwidth = 10000000.0;
    this.name = "MALink" + net.objNo[net.MALINK]++;
    net.nodes.addElement(this);
    net.malinks.addElement(this);
  }
  public MALink(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    av = new AttrValue(st);
    try {
      Double r = Double.valueOf(av.value);
      bandwidth = r.doubleValue();
      // System.out.println(r);
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    av = new AttrValue(st);
    w = Integer.parseInt(av.value);
    av = new AttrValue(st);
    h = Integer.parseInt(av.value);
    localTime = 0.0;
    net.nodes.addElement(this);
    net.malinks.addElement(this);
    net.objNo[type]++;
    //System.out.println(toString());
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " bandwidth="+bandwidth+
      " x="+x+" y="+y+" w="+w+" h="+h+
      nodeListToString()+linkListToString()+"\n";
  }
  public String nodeListToString() {
    String list= " nodeList=[";
    if (nodeList.size() == 0) return list += "]";
    for (int i=0; i<nodeList.size(); i++) {
      Node n = (Node) nodeList.elementAt(i);
      if (i==nodeList.size()-1) list += n.ID + "]";
      else list += n.ID + ",";
    }             
    return list;
  }
  public String nodeListNameToString() {
    String list = "[";
    for (int i=0; i<nodeList.size(); i++) {
      Node n = (Node) nodeList.elementAt(i);
      if (i==nodeList.size()-1) list += n.name + "]";
      else list += n.name + ",";
    }
    return list;
  }
  public void addNode(Node n) {
    nodeList.addElement(n);
  }
  
  public boolean inLink(int x, int y) {
      if ((this.x < x && (this.x+w) > x) && (this.y < y && (this.y+h)>y)){
        return true;
      } else {
        return false;
      }
  }
  public void eventHandle(EventRecord er) {
    // String msgType = er.getField("msgType");
    // don't care about msg type '
    // just forward it

      double queueingTime = localTime - er.time;
      if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
      }
      totalQueueingTime += queueingTime;
      noOfRequests++;


      // Find next node on net.pred[][]
      Link l = net.getNextLink(ID, er.dst);
      Node dstNode = net.searchNode(er.dst);
      Node srcNode = net.searchNode(er.src);
      if (l == null) {

        if (dstNode != null)
          net.println("This malink"+name+" is not connected to the "+dstNode.name);
        else
          net.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;

      // calculate the msgProcessing/transmission/propagation time
      /*  Chow: The transmission/propagation/msgProcessing times are computed
          by the sender to this malink  and should not be counted twice
          Only the queueing time should be added
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double transmissionTime = msgSize/l.bandwidth;
      double switchingTime = msgSize/bandwidth;
      //double msgProcessingTime = net.msgProcessingTime/power;
      double msgProcessingTime = (net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      double time = localTime+msgProcessingTime+switchingTime+transmissionTime;
      localTime = time;

      double propagationTime = l.distance/net.signalPropagationSpeed;
      time += propagationTime;
      */
     try {
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double propagationTime = l.distance/net.signalPropagationSpeed;

      er.time = localTime+propagationTime;
      er.next = nextNode.ID;

      // er.dst er.src no change
      d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();

      accQueueingTime += queueingTime;
      er.addField("queueingTime", ""+accQueueingTime);
      /*
      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += msgProcessingTime;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();
      accTransmissionTime += transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);
      */

      er.addField("route-qt", er.getField("route-qt")+name+":"+queueingTime+", ");

      String cds = er.getField("componentDelay");
      String malinkCDS = " "+ID+":QT="+queueingTime+",PT="+propagationTime;
      er.addField("componentDelay", cds+malinkCDS);
      net.logPrintln("***MALINK CDS:"+malinkCDS);
      // generate animation event if the animation flag is set on
      int p1x, p1y, p2x, p2y;
      if (net.animationOn) {  // here we assume next node will not be MALINK
        p1x = l.x; p1y = l.y;
        p2x = nextNode.x+16; p2y = nextNode.y+16;   
        net.animate(er, localTime, propagationTime, p1x, p1y, p2x, p2y);
        if (net.debug) System.err.println("MALINK forward reqID="+er.getField("reqID"));
      }
      net.el.queueEvent(er);
      net.println("time="+localTime+" name="+name+
          " forward reqID="+er.getField("reqID")+" msgSize="+msgSize+
          " queueingTime="+queueingTime);
     } catch (NumberFormatException e) {
      System.err.println("MALink: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
     }

  }
}
