//File:         MapArea.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Canvas Window with code for drawing animating messages
//              redraw network topology when window got damaged or reopen
//

package NetLayout;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;

// public class MapArea extends Canvas {
public class MapArea extends JPanel {
  Net net;
  Hashtable msgList; // list of animated msgs
  final int DISPRIGHT = 7;

  public MapArea() {
  }
  public MapArea(Net net) {
    this.net = net;
    msgList = new Hashtable();
  }
  public void redo() {
    Graphics g = getGraphics();
    //System.err.print("MapArea/redo()");
    paint(g);
  }
  public void addMsg(String reqID, EventRecord er) {
    msgList.put(reqID, er);
    if (net.debug) System.err.println("Animation: addMsg with reqID="+reqID);
  }
  public void removeMsg(String reqID) {
    msgList.remove(reqID);
  }
  public void drawCable(Graphics g, Cable c) {
      g.setColor(Color.green);
      if (c.n2.type == net.MALINK) g.drawLine(c.n1.getX()+16, c.n1.getY()+16, c.x, c.y);
      else g.drawLine(c.n1.getX()+16, c.n1.getY()+16, c.n2.getX()+16, c.n2.getY()+16);
  }
  public void drawPTPLink(Graphics g, PTPLink l) {
      g.setColor(Color.red);
      g.drawLine(l.n1.getX()+16, l.n1.getY()+16,
                     l.n2.getX()+16, l.n2.getY()+16);
  }
  public void drawMALink(Graphics g, MALink l) {
      g.setColor(Color.green);
      g.fillRect(l.x, l.y, l.w, l.h);
  }
  public void paint(Graphics g) {
  //public void paintComponent(Graphics g) {
  //super.paintComponent(g);

    int w, h;
    Graphics2D g2 = (Graphics2D) g;
    Dimension d = getSize();
    w = d.width;
    h = d.height;
    BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
    Graphics2D big = bi.createGraphics();
    big.setColor(Color.lightGray);
    big.clearRect(0, 0, w, h);
    for (int i=0; i<net.nodes.size(); i++) {
      Node n = (Node) net.nodes.elementAt(i);
      // while (!g.drawImage(net.f.im[type], x, y, net.f)) {};
      if (n.type != net.MALINK) {
        while (!big.drawImage(net.f.im[n.type], n.x, n.y, net.f)) {};
        big.drawString(n.name, n.x, n.y+44);
      }
    }
    for (int i=0; i<net.malinks.size(); i++) {
      MALink l = (MALink) net.malinks.elementAt(i);
      drawMALink(big, l);
    }
    for (int i=0; i< net.links.size(); i++) {
      Link l = (Link) net.links.elementAt(i);
      if (l.type == net.CABLE) {
        Cable c = (Cable) l;
        drawCable(big, c);
      } else if (l.type == net.PTPLINK) {
        PTPLink ptpl = (PTPLink) l;
        drawPTPLink(big, ptpl);
      }
    }
    Enumeration keys = msgList.keys();
    if (net.debug) System.err.println("Animation: get keys");
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      if (net.debug) System.err.println("Animation: get key="+key);
      EventRecord er = (EventRecord) msgList.get(key);
      drawAnimateMsg(big, er);
    }
    g2.drawImage(bi, 0, 0, this);

  }
  public void drawAnimateMsg(Graphics2D big, EventRecord er) {
    int p1x = Integer.parseInt(er.getField("p1x"));
    int p2x = Integer.parseInt(er.getField("p2x"));
    int p1y = Integer.parseInt(er.getField("p1y"));
    int p2y = Integer.parseInt(er.getField("p2y"));
    int step = Integer.parseInt(er.getField("step"));
    Point p1 = new Point(p1x, p1y);
    Point p2 = new Point(p2x, p2y);
    Point p  = new Point(p1.x+(p2.x-p1.x)*step/net.animationMaxSteps,
                         p1.y+(p2.y-p1.y)*step/net.animationMaxSteps);
    String msgType = er.getField("msgType");
    double rad = Math.atan((p2.y-p1.y)/(double) (p2.x-p1.x));
    String reqID = er.getField("reqID");
    if (net.debug) System.err.println("Animation: reqID="+reqID+", p1x="+p1x+", p1y="+p1y+
                       ", p2x="+p2x+", p2y="+p2y);

    int i=0;
    TextLayout msgTl;
    AffineTransform saveXform;
    AffineTransform at = new AffineTransform();
    big.setStroke(new BasicStroke(1.0f));
    String src = net.searchNode(er.src).getName();
    String dst = net.searchNode(er.dst).getName();
    msgTl = new TextLayout(msgType+" | "+src+" | "+dst+" | "+reqID, new Font("Helvetica", 1, 16), new FontRenderContext(null, false, false));
    Rectangle2D bounds = msgTl.getBounds();
    int height = (int) bounds.getHeight();
    int DISPLEFT = -(3+height); // make it relative to the font height
    Rectangle rec = new Rectangle(-1, -height+1, (int) bounds.getWidth()+3, height+2);
    AffineTransform atMsg = new AffineTransform();   // for msg text
    //AffineTransform atMsbig = new AffineTransform();
    AffineTransform atMsg3 = new AffineTransform();  // for rectangle box around msg text

    // atMsg.translate(p1.x, p1.y+(float)msgTl.getBounds().getHeight());
    int displacement = DISPRIGHT;
    if (p1.x == p2.x && p1.y > p2.y || p1.x > p2.x) displacement = DISPLEFT;

    atMsg.translate(p.x+displacement*Math.sin(rad),
                    p.y-displacement*Math.cos(rad));
    atMsg.rotate(rad);
    saveXform = big.getTransform();
    atMsg3.setToIdentity();
    atMsg3.translate(p.x+displacement*Math.sin(rad),
                     p.y-displacement*Math.cos(rad));
    atMsg3.rotate(rad);
    big.setTransform(atMsg3);
    if (msgType.compareTo("http-request")==0) {
      big.setPaint(Color.cyan);
      big.fill(rec);
      big.setPaint(Color.blue);
      big.draw(rec);
    } else {
      big.setPaint(Color.pink);
      big.fill(rec);
      big.setPaint(Color.red);
      big.draw(rec);
    }

    big.setTransform(saveXform);
    big.setPaint(Color.black);
    big.draw(msgTl.getOutline(atMsg));
    big.fill(msgTl.getOutline(atMsg));
  }
  /*
  public void repaint(Graphics g) {
    paint(g);
  }
  */
  public void update(Graphics g) {
    paint(g);
  }
}
