//File:         ProbingResponder.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              3/9/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Class for respond probing request
//              Add unidirectional probe code later
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector

public class ProbingResponder extends Node {

  public ProbingResponder(Net net, int x, int y) {
    this.x = x;
    this.y = y;
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    type = net.PROBERCV;
    name =  "resp" + net.objNo[type]++;

    net.nodes.addElement(this);
  }
  public ProbingResponder(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    net.objID++;
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    try {
      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      power = r.doubleValue();
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    // System.out.println(toString());

    localTime = 0.0;
    net.nodes.addElement(this);
    net.objNo[type]++;
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+
      " lon="+lon+" lat="+lat+" power="+power+
      linkListToString()+
      "\n";
  }
  public void eventHandle(EventRecord er) {

    double queueingTime = localTime - er.time;
    if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
    }
    totalQueueingTime += queueingTime;
    noOfRequests++;

    String msgType = er.getField("msgType");
    if (msgType.compareTo("probing-request") == 0) {
      if (localTime < er.time) localTime = er.time;

     try {
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double msgProcessingTime=(net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime;

      double msgProcessingTime2=(net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime2;
      // should consider different probing response msg size later

      // if the outgoing response msg has difference size than we need to
      // modify the above formula

      // put receiveTimeStamp in the probing-response
      er.addField("rcvTimeStamp", ""+localTime);

      // Find return link
      Link l = net.getNextLink(ID, er.src);
      if (l == null) {
        Node n = net.searchNode(er.dst);
        if (n != null)
          net.println("This ProbingResponder "+name+" is not connected to the "+n.name);
        else
          net.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;


      double transmissionTime = msgSize/l.bandwidth;

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, localTime);
      }

      er.time = time;
      er.next = nextNode.ID;
      er.dst = er.src;
      er.src = ID;
      er.addField("msgType", "probing-response");
      // put receiveTimeStamp in the probing-response
      er.addField("rcvTimeStamp", ""+localTime);

      d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();
      accQueueingTime += queueingTime;
      er.addField("queueingTime", ""+accQueueingTime);

      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += 2*msgProcessingTime;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();
      accTransmissionTime += transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);

      d = Double.valueOf(er.getField("propagationTime"));
      double accPropagationTime = d.doubleValue();


      accPropagationTime += propagationTime;
      er.addField("propagationTime", ""+accPropagationTime);

      er.addField("route-qt", er.getField("route-qt")+name+":"+queueingTime+", ");

      String cds = er.getField("componentDelay");
      String ProbingResponderCDS = " "+ID+":QT="+queueingTime+",MPT="+(msgProcessingTime+msgProcessingTime2)+",RT="+0.0;
      String linkCDS = " "+l.ID+":PT="+propagationTime+",TT="+transmissionTime;
      er.addField("componentDelay", cds+ProbingResponderCDS+linkCDS);


      // generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = l.n2.x+16; p2y = l.n2.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }
      net.el.queueEvent(er);
      net.println("time="+localTime+" name="+name+
          " return reqID="+er.getField("reqID")+" msgSize="+msgSize+
          " queueingTime="+queueingTime);
     } catch (NumberFormatException e) {
      System.err.println("Serve.java: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
     }
    }
  }
}


