//File:         Server.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Model Server behavior
//              With Retrieve Time simulate file system speed
//              Model the web document size distribution
//

package NetLayout;

import java.util.*; // for StringTokenizer and Vector

public class Server extends Node {
  Stat reqDocSt;
  public Server(Net net, int x, int y) {
    this.x = x;
    this.y = y;
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    type = net.SERVER;
    name =  "server" + net.objNo[type]++;
    reqDocSt = new Stat();
    reqDocSt.avg = 40000; reqDocSt.min = 40000; reqDocSt.max = 40000;
    net.nodes.addElement(this);
  }
  public Server(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    net.objID++;
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    try {
      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      power = r.doubleValue();
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    // System.out.println(toString());
    reqDocSt = new Stat();
    reqDocSt.avg = 40000; reqDocSt.min = 40000; reqDocSt.max = 40000;
    localTime = 0.0;
    net.nodes.addElement(this);
    net.objNo[type]++;
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+
      " lon="+lon+" lat="+lat+" power="+power+
      linkListToString()+
      "\n";
  }
  public void eventHandle(EventRecord er) {
    EventRecord ner;
    double queueingTime = localTime - er.time;
    if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
    }
    totalQueueingTime += queueingTime;
    noOfRequests++;

    String msgType = er.getField("msgType");
    if (msgType.compareTo("http-request") == 0) {
      if (localTime < er.time) localTime = er.time;

     try {
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double msgProcessingTime=(net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime;

      // retrieve document
      int docIndex = Integer.parseInt(er.getField("docIndex"));
      double docSize = ((0.0+docIndex)/net.noOfDocs )*(reqDocSt.max-reqDocSt.min)+reqDocSt.min;

      // calculate the retrieval time
      double retrieveTime = docSize/(power*net.retrieveSpeed);
      // we may consider random service time for file retrieval
      localTime += retrieveTime;


      // Find return link
      Link l = net.getNextLink(ID, er.src);
      if (l == null) {
        Node n = net.searchNode(er.dst);
        if (n != null)
          net.println("This server "+name+" is not connected to the "+n.name);
        else
          net.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;

     int n = (int) Math.ceil(docSize/net.MTU);
     double lastPacketSize = docSize%net.MTU;
     if (lastPacketSize <= 512) lastPacketSize = 512;
     double packetSize = net.MTU;


     int dst = er.src;
     int src = ID;
     int next = nextNode.ID;
     String startTime = er.getField("startTime");
     String reqIDstr = er.getField("reqID");
     System.out.println("Server ID="+ID+"docSize="+docSize+", #packets="+n+", MTU="+net.MTU+
                        ", lastPacketSize="+ lastPacketSize+
                        ", next ="+next);
     for (int i=0; i<n; i++) {
      if (i == n-1) packetSize = lastPacketSize;
      double msgProcessingTime2=(net.msgProcessingTime+packetSize/net.msgProcessingSpeed)/power;
      localTime += msgProcessingTime2;

      double transmissionTime = packetSize/l.bandwidth;

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, packetSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, packetSize, localTime);
      }

     if (i == 0) {
      er.time = time;
      er.next = nextNode.ID;
      er.dst = er.src;
      er.src = ID;
      er.addField("msgType", "http-response");
      er.addField("msgSize", ""+packetSize);
      er.addField("retrieveTime", ""+retrieveTime);
      er.addField("pkt#", ""+i);
      d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();
      accQueueingTime += queueingTime;
      er.addField("queueingTime", ""+accQueueingTime);

      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += msgProcessingTime+msgProcessingTime2;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();
      accTransmissionTime += transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);

      d = Double.valueOf(er.getField("propagationTime"));
      double accPropagationTime = d.doubleValue();


      accPropagationTime += propagationTime;
      er.addField("propagationTime", ""+accPropagationTime);

      er.addField("route-qt", er.getField("route-qt")+name+":"+queueingTime+", ");

      String cds = er.getField("componentDelay");
      String serverCDS = " "+ID+":QT="+queueingTime+",MPT="+(msgProcessingTime+msgProcessingTime2)+",RT="+retrieveTime;
      String linkCDS = " "+l.ID+":PT="+propagationTime+",TT="+transmissionTime;
      er.addField("componentDelay", cds+serverCDS+linkCDS);
// generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = l.n2.x+16; p2y = l.n2.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }
      net.el.queueEvent(er);
      System.out.println("time="+localTime+" name="+name+
          " return reqID="+er.getField("reqID")+" pkt#="+i+" msgSize="+packetSize+
          " queueingTime="+queueingTime);
     } else {

      ner = new EventRecord(time, ID, ID, ID); // first msg use the same ER
      ner.time = time;
      ner.next = next;
      ner.dst = dst;
      ner.src = src;
      ner.addField("startTime", startTime);
      ner.addField("msgType", "http-response");
      ner.addField("msgSize", ""+packetSize);
      ner.addField("retrieveTime", ""+retrieveTime);
      ner.addField("pkt#", ""+i);
      ner.addField("reqID", reqIDstr);
      ner.addField("queueingTime", "0.0");
      ner.addField("msgProcessingTime", ""+msgProcessingTime2);
      ner.addField("transmissionTime", ""+transmissionTime);
      ner.addField("propagationTime", ""+propagationTime);
      ner.addField("route-qt", "0.0");
      String serverCDS = " "+ID+":QT=0.0,MPT="+msgProcessingTime2+",RT=0.0";
      String linkCDS = " "+l.ID+":PT="+propagationTime+",TT="+transmissionTime;
      ner.addField("componentDelay", serverCDS+linkCDS);
      // generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = l.n2.x+16; p2y = l.n2.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }
      net.el.queueEvent(ner);
      System.out.println("time="+localTime+" name="+name+
          " return reqID="+ner.getField("reqID")+" pkt#="+i+" msgSize="+packetSize+
          " queueingTime="+queueingTime);

     }


     }
    } catch (NumberFormatException e) {
      System.err.println("Serve.java: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
    }
   }
  }
}


