//File:         SimControlProperty.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              3/9/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Dialog Window for specify the simulation control parameters
//              including starting probing time, probing period, available
//              bandwidth report period, probing method parameters,
//              traffic type selection, plot parameters
//

package NetLayout;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class SimControlProperty extends Dialog implements ActionListener {
  final static int size = 20;

  NetLayoutFrame f;
  Net net;

  JPanel panel1 = new JPanel();

  JPanel panel3 = new JPanel();
  JPanel insetsPanel1 = new JPanel();

  JPanel insetsPanel3 = new JPanel();
  JSeparator sep1 = new JSeparator();
  JSeparator sep2 = new JSeparator();
  JSeparator sep3 = new JSeparator();
  Label statusBar = new Label();

  JButton button1 = new JButton();
  JButton button2 = new JButton();
  JLabel label1 = new JLabel();
  JLabel label2 = new JLabel();

  JLabel simRunNameLabel = new JLabel("SimulationRunName:", JLabel.RIGHT);
  JTextField simRunNameField;

  JLabel startProbeTimeLabel = new JLabel("StartProbeTime:", JLabel.RIGHT);
  JTextField startProbeTimeField;

  JLabel probingPeriodLabel = new JLabel("ProbingPeriod:", JLabel.RIGHT);
  JTextField probingPeriodField;

  JLabel abwReportPeriodLabel = new JLabel("ABWReportPeriod:", JLabel.RIGHT);
  JTextField abwReportPeriodField;

  JLabel probingMsgSizeLabel = new JLabel("ProbingMsgSize:", JLabel.RIGHT);
  JTextField probingMsgSizeField;

  JLabel noOfMsgsPerIterationLabel = new JLabel("NoOfMsgsPerIteration:", JLabel.RIGHT);
  JTextField noOfMsgsPerIterationField;


  JLabel methodLabel = new JLabel("Select the probing method:", JLabel.LEFT);
  JCheckBox timeGapDivergeCheckBox = new JCheckBox("Time Gap Diverge Method");
  JCheckBox equalBWTimeGapCheckBox = new JCheckBox("Equal BW Time Gap");

  JLabel divergeThreshholdLabel = new JLabel("divergeThreshhold:", JLabel.RIGHT);
  JTextField divergeThreshholdField;
  JLabel maxIterationLabel = new JLabel("maxIteration:", JLabel.RIGHT);
  JTextField maxIterationField;
  //added by Jihui
  JLabel accuracyReqLabel = new JLabel("accuracyRequirement:", JLabel.RIGHT);
  JTextField accuracyReqField;
  JLabel bwHighLabel = new JLabel("bwHigh:", JLabel.RIGHT);
  JTextField bwHighField;
  JLabel bwLowLabel = new JLabel("bwLow:", JLabel.RIGHT);
  JTextField bwLowField;

  JCheckBox cProbeCheckBox = new JCheckBox("CProbe Method");
  JCheckBox roundTripDelayDifferenceCheckBox = new JCheckBox("Round Trip Delay Difference Method");

  JLabel trafficLabel = new JLabel("Select the traffic type:", JLabel.LEFT);
  JCheckBox constantBitRateCheckBox = new JCheckBox("Constant Bit Rate");
  JCheckBox constantClientPacketCheckBox = new JCheckBox("Constant Client Packet");
  JCheckBox randomWebTrafficCheckBox = new JCheckBox("Random Web Traffic");
  JCheckBox selfSimilarTrafficCheckBox = new JCheckBox("Self Similar Traffic");

  JLabel plotParameterLabel = new JLabel("Plot Parameters:", JLabel.LEFT);
  JLabel maxBWLabel = new JLabel("maxBW:", JLabel.RIGHT);
  JTextField maxBWField;
  JLabel minBWLabel = new JLabel("minBW:", JLabel.RIGHT);
  JTextField minBWField;
  JLabel startPlotTimeLabel = new JLabel("startPlotTime:", JLabel.RIGHT);
  JTextField startPlotTimeField;
  JLabel pixelsPerSecondLabel = new JLabel("pixelsPerSecond:", JLabel.RIGHT);
  JTextField pixelsPerSecondField;

  BorderLayout borderLayout1 = new BorderLayout();

  BorderLayout borderLayout3 = new BorderLayout();
  FlowLayout flowLayout1 = new FlowLayout();

  String product = "ProbSim: Probing Simulator Tool Copyright (c) 2000";
  String dialogName = "Simulation Control Property";

  public SimControlProperty(Frame parent, Net n) {
    super(parent);
    f = (NetLayoutFrame) parent;
    net = n;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    pack();
  }

  private void jbInit() throws Exception  {
    this.setTitle("Simulation Control Property");
    setResizable(false);
    panel1.setLayout(borderLayout1);

    panel3.setLayout(borderLayout3);
    insetsPanel1.setLayout(flowLayout1);

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    insetsPanel3.setLayout(gbl);

    label1.setText(product);
    label2.setText(dialogName);

    button1.setText("Set Simulation Parameters");
    button1.addActionListener(this);
    button2.setText("Cancel");
    button2.addActionListener(this);
    insetsPanel1.add(button1, null);
    insetsPanel1.add(button2, null);
    panel1.add(insetsPanel1, BorderLayout.SOUTH);


    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(label1, gbc);
    insetsPanel3.add(label1);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(label2, gbc);
    insetsPanel3.add(label2);

    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep1, gbc);
    insetsPanel3.add(sep1);
//    gbc.fill = 0;

    String srname = net.fileName;
    simRunNameField = new JTextField(srname, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(simRunNameLabel, gbc);
    insetsPanel3.add(simRunNameLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(simRunNameField, gbc);
    insetsPanel3.add(simRunNameField);

    String s = ""+ net.startProbeTime;
    startProbeTimeField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(startProbeTimeLabel, gbc);
    insetsPanel3.add(startProbeTimeLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(startProbeTimeField, gbc);
    insetsPanel3.add(startProbeTimeField);

    s = ""+net.probingPeriod;
    probingPeriodField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(probingPeriodLabel, gbc);
    insetsPanel3.add(probingPeriodLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(probingPeriodField, gbc);
    insetsPanel3.add(probingPeriodField);

    s = ""+net.abwReportPeriod;
    abwReportPeriodField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(abwReportPeriodLabel, gbc);
    insetsPanel3.add(abwReportPeriodLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(abwReportPeriodField, gbc);
    insetsPanel3.add(abwReportPeriodField);

    s = ""+net.probingMsgSize;
    probingMsgSizeField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(probingMsgSizeLabel, gbc);
    insetsPanel3.add(probingMsgSizeLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(probingMsgSizeField, gbc);
    insetsPanel3.add(probingMsgSizeField);

    s = ""+net.noOfMsgsPerIteration;
    noOfMsgsPerIterationField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(noOfMsgsPerIterationLabel, gbc);
    insetsPanel3.add(noOfMsgsPerIterationLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(noOfMsgsPerIterationField, gbc);
    insetsPanel3.add(noOfMsgsPerIterationField);


    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(methodLabel, gbc);
    insetsPanel3.add(methodLabel);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    timeGapDivergeCheckBox.setSelected(true);
    gbl.setConstraints(timeGapDivergeCheckBox, gbc);
    insetsPanel3.add(timeGapDivergeCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,40,0,0);
    equalBWTimeGapCheckBox.setSelected(true);
    gbl.setConstraints(equalBWTimeGapCheckBox, gbc);
    insetsPanel3.add(equalBWTimeGapCheckBox);

    s = ""+net.divergeThreshhold;
    divergeThreshholdField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(divergeThreshholdLabel, gbc);
    insetsPanel3.add(divergeThreshholdLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(divergeThreshholdField, gbc);
    insetsPanel3.add(divergeThreshholdField);

    s = ""+net.maxIteration;
    maxIterationField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(maxIterationLabel, gbc);
    insetsPanel3.add(maxIterationLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(maxIterationField, gbc);
    insetsPanel3.add(maxIterationField);

    s = ""+net.accuracyReq;
    accuracyReqField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(accuracyReqLabel, gbc);
    insetsPanel3.add(accuracyReqLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(accuracyReqField, gbc);
    insetsPanel3.add(accuracyReqField);

    s = ""+net.bwHigh;
    bwHighField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(bwHighLabel, gbc);
    insetsPanel3.add(bwHighLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(bwHighField, gbc);
    insetsPanel3.add(bwHighField);

    s = ""+net.bwLow;
    bwLowField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(bwLowLabel, gbc);
    insetsPanel3.add(bwLowLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(bwLowField, gbc);
    insetsPanel3.add(bwLowField);



    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    cProbeCheckBox.setSelected(false);
    gbl.setConstraints(cProbeCheckBox, gbc);
    insetsPanel3.add(cProbeCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    roundTripDelayDifferenceCheckBox.setSelected(false);
    gbl.setConstraints(roundTripDelayDifferenceCheckBox, gbc);
    insetsPanel3.add(roundTripDelayDifferenceCheckBox);


    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(trafficLabel, gbc);
    insetsPanel3.add(trafficLabel);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    constantBitRateCheckBox.setSelected(false);
    gbl.setConstraints(constantBitRateCheckBox, gbc);
    insetsPanel3.add(constantBitRateCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    constantClientPacketCheckBox.setSelected(true);
    gbl.setConstraints(constantClientPacketCheckBox, gbc);
    insetsPanel3.add(constantClientPacketCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    randomWebTrafficCheckBox.setSelected(false);
    gbl.setConstraints(randomWebTrafficCheckBox, gbc);
    insetsPanel3.add(randomWebTrafficCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,20,0,0);
    selfSimilarTrafficCheckBox.setSelected(false);
    gbl.setConstraints(selfSimilarTrafficCheckBox, gbc);
    insetsPanel3.add(selfSimilarTrafficCheckBox);

    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(plotParameterLabel, gbc);
    insetsPanel3.add(plotParameterLabel);

    s = ""+net.maxBW;
    maxBWField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(maxBWLabel, gbc);
    insetsPanel3.add(maxBWLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(maxBWField, gbc);
    insetsPanel3.add(maxBWField);

    s = ""+net.minBW;
    minBWField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(minBWLabel, gbc);
    insetsPanel3.add(minBWLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(minBWField, gbc);
    insetsPanel3.add(minBWField);


    s = ""+net.startPlotTime;
    startPlotTimeField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(startPlotTimeLabel, gbc);
    insetsPanel3.add(startPlotTimeLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(startPlotTimeField, gbc);
    insetsPanel3.add(startPlotTimeField);

    s = ""+net.pixelsPerSecond;
    pixelsPerSecondField = new JTextField(s, size);
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridwidth = 1;
    gbc.insets = new Insets(0,0,0,0);
    gbl.setConstraints(pixelsPerSecondLabel, gbc);
    insetsPanel3.add(pixelsPerSecondLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(pixelsPerSecondField, gbc);
    insetsPanel3.add(pixelsPerSecondField);


    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep2, gbc);
    insetsPanel3.add(sep2);


    gbc.anchor = GridBagConstraints.SOUTH;
    JLabel statusBarLabel = new JLabel("status:");
    gbc.gridwidth = 1;
    gbl.setConstraints(statusBarLabel, gbc);
    insetsPanel3.add(statusBarLabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(statusBar, gbc);
    insetsPanel3.add(statusBar);

    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.insets = new Insets(0,0,10,0);
    gbl.setConstraints(sep3, gbc);
    insetsPanel3.add(sep3);


    panel3.add(insetsPanel3,BorderLayout.CENTER);
    panel1.add(panel3, BorderLayout.CENTER);
    this.add(panel1, null);
  }

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  void cancel() {
    dispose();
  }
  boolean startSimulation() {
    net.simRunName = simRunNameField.getText();
    String ns;

    try {
      ns = startProbeTimeField.getText();
      Double r = Double.valueOf(ns);
      net.startProbeTime = r.doubleValue();
      ns = probingPeriodField.getText();
      r = Double.valueOf(ns);
      net.probingPeriod = r.doubleValue();
      ns = abwReportPeriodField.getText();
      r = Double.valueOf(ns);
      net.abwReportPeriod = r.doubleValue();

      net.probingMsgSize = Integer.parseInt(probingMsgSizeField.getText());

      net.noOfMsgsPerIteration = Integer.parseInt(noOfMsgsPerIterationField.getText());

      ns = accuracyReqField.getText();
      r = Double.valueOf(ns);
      net.accuracyReq = r.doubleValue();

      System.out.println("net.accReq in SimControl is"+net.accuracyReq);

      net.maxIteration = Integer.parseInt(maxIterationField.getText());
      //By Jihui
 //     net.accuracyReq = Integer.parseInt(accuracyReqField.getText());
      ns = divergeThreshholdField.getText();
      r = Double.valueOf(ns);
      net.divergeThreshhold = r.doubleValue();

      ns = bwHighField.getText();
      r = Double.valueOf(ns);
      net.bwHigh = r.doubleValue();

      ns = bwLowField.getText();
      r = Double.valueOf(ns);
      net.bwLow = r.doubleValue();

      if (net.bwLow <= 0.0) {
         statusBar.setText("!!bwLow can not be <= 0.0. suggest 20000 !! ");
         return false;
      }

      ns = maxBWField.getText();
      r = Double.valueOf(ns);
      net.maxBW = r.doubleValue();
      ns = minBWField.getText();
      r = Double.valueOf(ns);
      net.minBW = r.doubleValue();
      ns = startPlotTimeField.getText();
      r = Double.valueOf(ns);
      net.startPlotTime = r.doubleValue();
      net.pixelsPerSecond = Integer.parseInt(pixelsPerSecondField.getText());
    } catch (NumberFormatException e) {
      statusBar.setText("!!Could not convert string to double/int!! ");
      return false;
    }
    if (timeGapDivergeCheckBox.isSelected()) net.probingMethod = net.TimeGapDivergeMethod;
    if (equalBWTimeGapCheckBox.isSelected()) net.equalBWTimeGap = true;
    else  net.equalBWTimeGap = false;
    if (cProbeCheckBox.isSelected()) net.probingMethod = net.CProbeMethod;
    if (roundTripDelayDifferenceCheckBox.isSelected()) net.probingMethod = net.RoundTripDelayDifferenceMethod;

    if (constantBitRateCheckBox.isSelected()) net.trafficType = net.ConstantBitRate;
    if (constantClientPacketCheckBox.isSelected()) net.trafficType = net.ConstantClientPacket;
    if (randomWebTrafficCheckBox.isSelected()) net.trafficType = net.RandomWebTraffic;
    if (selfSimilarTrafficCheckBox.isSelected()) net.trafficType = net.SelfSimilarTraffic;

    return true;

  }
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == button1) {
       if (startSimulation()) {
          net.plotProbingResults = new PlotProbingResults(net.minBW, net.maxBW, net.startPlotTime, net.pixelsPerSecond);
          Dimension frmSize = net.f.getSize();
          Point loc = net.f.getLocation();
          net.plotProbingResults.setLocation(loc.x+frmSize.width, loc.y);
          dispose();
       }
    }
    if (e.getSource() == button2) {
      cancel();
    }
  }
}

