//File:         Switch.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Model Switch Behavior
//

package NetLayout;

import java.io.*;
import java.util.*; // for StringTokenizer and Vector

public class Switch extends Node {                          
  protected Vector nodeList = new Vector (2,2);
  double bandwidth;
  public Switch(Net net, int x, int y) {
    this.x = x;
    this.y = y;
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    type = net.SWITCH;
    bandwidth = 1.0E+8;
    name =  "switch" + net.objNo[type]++;
    net.nodes.addElement(this);     // add to net node list
    net.switches.addElement(this);
  }
  public Switch(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;
    AttrValue av = new AttrValue(st);
    name = av.value;
    av = new AttrValue(st);
    ID = Integer.parseInt(av.value);
    net.addComponentDelay(ID);
    net.objID++;
    av = new AttrValue(st);
    x = Integer.parseInt(av.value);
    av = new AttrValue(st);
    y = Integer.parseInt(av.value);
    try {
      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      bandwidth = r.doubleValue();
    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to double " + av.value);
    }
    // System.out.println(toString());
    localTime = 0.0;
    net.nodes.addElement(this);     // add to net node list
    net.switches.addElement(this);
    net.objNo[type]++;
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+
      " lon="+lon+" lat="+lat+" bandwidth="+bandwidth+
      nodeListToString()+linkListToString()+
      "\n";
  }
  public String nodeListToString() {
    String list= " nodeList=[";
    if (nodeList.size() == 0) return list += "]";
    for (int i=0; i<nodeList.size(); i++) {
      Node n = (Node) nodeList.elementAt(i);
      if (i==nodeList.size()-1) list += n.ID + "]";
      else list += n.ID + ",";
    }          
    return list;
  }
  public String nodeListNameToString() {
    String list= " nodeList=[";
    if (nodeList.size() == 0) return list += "]";
    for (int i=0; i<nodeList.size();i++) {
      Node n = (Node) nodeList.elementAt(i);
      if (i==nodeList.size()-1) list += n.name + "]";
      else list += n.name + ",";
    }
    return list;
  }
  public void addNode(Node n) {
    nodeList.addElement(n);
  }

  public void eventHandle(EventRecord er) {
    //String msgType = er.getField("msgType");
    // don't care about msg type '
    // just forward it
      double queueingTime = localTime - er.time;
      if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
      }
      totalQueueingTime += queueingTime;
      noOfRequests++;


      // Find next node on net.pred[][]
      Link l = net.getNextLink(ID, er.dst);
      Node dstNode = net.searchNode(er.dst);
      Node srcNode = net.searchNode(er.src);
      if (l == null) {

        if (dstNode != null)
          net.println("This malink"+name+" is not connected to the "+dstNode.name);
        else
          net.println("Incorrect er.dst as ID. er.dst="+er.dst);
        return;
      }
      Node nextNode = (l.n1 == this) ? l.n2 : l.n1;

      // calculate the msgProcessing/transmission/propagation time
     try {
      Double d = Double.valueOf(er.getField("msgSize"));
      double msgSize = d.doubleValue();
      double transmissionTime = msgSize/l.bandwidth;
      double switchingTime = msgSize/bandwidth;
      //double msgProcessingTime = net.msgProcessingTime/power;
      double msgProcessingTime = (net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;


      localTime += msgProcessingTime+switchingTime;
      // to model parallel activity we need to improve the above localTime calculation
      // Here we assume output and input port parallel processing
      //                localTime only reflects the queueing time

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, localTime);
      }

      er.time = time;
      er.next = nextNode.ID;
      // er.src = ID;
      // er.dst no change
      d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();
      accQueueingTime += queueingTime;
      er.addField("queueingTime", ""+accQueueingTime);

      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += msgProcessingTime+switchingTime;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();
      accTransmissionTime += transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);

      d = Double.valueOf(er.getField("propagationTime"));
      double accPropagationTime = d.doubleValue();



      accPropagationTime += propagationTime;
      er.addField("propagationTime", ""+accPropagationTime);

      er.addField("route-qt", er.getField("route-qt")+name+":"+queueingTime+", ");

      String cds = er.getField("componentDelay");
      String switchCDS = " "+ID+":QT="+queueingTime+",MPT="+(msgProcessingTime+switchingTime);
      String linkCDS = " "+l.ID+":PT="+propagationTime+",TT="+transmissionTime;
      er.addField("componentDelay", cds+switchCDS+linkCDS);


      // generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = nextNode.x+16; p2y = nextNode.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }
      net.el.queueEvent(er);
      net.println("time="+localTime+" name="+name+
          " forward reqID="+er.getField("reqID")+" msgSize="+msgSize+
          " queueingTime="+queueingTime);
     } catch (NumberFormatException e) {
      System.err.println("Switch.java: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
     }
  }
}


