//File:         TableProbingResult.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              class for display probing result tables
//              adapted from swing class example
//

package NetLayout;

import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.table.AbstractTableModel;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;

public class TableProbingResult extends JFrame {
    private boolean DEBUG = false;
    ProbingAgent pa;

    public TableProbingResult(ProbingAgent p, BWRange r) {
        super("Available Bandwidth Probing Result:BW within "+r.toString()+" original range="+p.curBWR.toString()+" MS="+p.msgSize+" #Msg="+p.noOfMsgs);
        pa = p;
        String result = "Round="+p.net.probingRound+", Iteration="+p.iteration+" Probing Bandwidth Result: BW within "+r.toString();
        JLabel range = new JLabel(result);
        getContentPane().add(range, BorderLayout.NORTH);

        MyTableModel myModel = new MyTableModel();

        myModel.setSize(p.sendingTime.length);
        boolean display = true;
        for (int i=1; i<= p.sendingTime.length-1; i++) {

            myModel.setValueAt(new Integer(i), i-1, 0);
            if (p.sendingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double(p.sendingTime[i]*1000), i-1, 1);
            else
               myModel.setValueAt(new Double(0), i-1, 1);
            if (display && p.receivingTime[i] <= p.iterationEndTime) {
               myModel.setValueAt(new Double(p.receivingTime[i]*1000), i-1, 2);
               if (p.receivingTime[i] == p.iterationEndTime) {
                  display = false;
               }
            } else
               myModel.setValueAt(new Double(0), i-1, 2);
            if (display && p.receivingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double((p.receivingTime[i]-p.sendingTime[i])*1000), i-1, 3);
            else if (p.receivingTime[i] == p.iterationEndTime) {
               myModel.setValueAt(new Double((p.receivingTime[i]-p.sendingTime[i])*1000), i-1, 3);
            } else
               myModel.setValueAt(new Double(0), i-1, 3);
            if (p.sendingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double(p.msgSize/p.sendingTimeGap[i]), i-1, 4);
            else
               myModel.setValueAt(new Double(0), i-1, 4);
            if (p.sendingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double( p.absSendingTimeGap[i]*1000 ), i-1, 5);
            else
               myModel.setValueAt(new Double(0), i-1, 5);
            if (display && p.receivingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double( p.absReceivingTimeGap[i]*1000 ), i-1, 6);
            else if (p.receivingTime[i] == p.iterationEndTime) {
               myModel.setValueAt(new Double( p.absReceivingTimeGap[i]*1000), i-1, 6);
            } else
               myModel.setValueAt(new Double(0), i-1, 6);
            if (display && p.receivingTime[i] <= p.iterationEndTime)
               myModel.setValueAt(new Double(p.absGapDifference[i]*1000), i-1, 7 );
            else if (p.receivingTime[i] == p.iterationEndTime) {
               myModel.setValueAt(new Double(p.absGapDifference[i]*1000), i-1, 7 );
            } else
               myModel.setValueAt(new Double(0), i-1, 7);
        }

        TableSorter sorter = new TableSorter(myModel); //ADDED THIS
        //JTable table = new JTable(myModel);          //OLD
        JTable table = new JTable(sorter);             //NEW
        sorter.addMouseListenerToHeaderInTable(table); //ADDED THIS
        table.setPreferredScrollableViewportSize(new Dimension(100, 200));

        //Create the scroll pane and add the table to it.
        JScrollPane scrollPane = new JScrollPane(table);

        //Add the scroll pane to this window.
        getContentPane().add(scrollPane, BorderLayout.CENTER);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }

    class MyTableModel extends AbstractTableModel {
        final String[] columnNames = {"Seq#",
                                "SendingTime",
                                "ReceivingTime",
                                "RTTime",
                                "BW Demand",
                                "STGap",
                                "RTGap",
                                "GapDiff"};
        Object[][] data;
        public void setSize(int row) {
          data = new Object[row][columnNames.length];
        }


        public Object getValueAt(int row, int col) {
            return data[row][col];
        }
        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return data.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }



        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * Don't need to implement this method unless your table's
         * editable.
         */
        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears onscreen.
            if (col < 2) {
                return false;
            } else {
                return true;
            }
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */
        public void setValueAt(Object value, int row, int col) {
            if (DEBUG) {
                System.out.println("Setting value at " + row + "," + col
                                   + " to " + value
                                   + " (an instance of "
                                   + value.getClass() + ")");
            }

            data[row][col] = value;
            fireTableCellUpdated(row, col);

            if (DEBUG) {
                System.out.println("New value of data:");
                printDebugData();
            }
        }

        private void printDebugData() {
            int numRows = getRowCount();
            int numCols = getColumnCount();

            for (int i=0; i < numRows; i++) {
                System.out.print("    row " + i + ":");
                for (int j=0; j < numCols; j++) {
                    System.out.print("  " + data[i][j]);
                }
                System.out.println();
            }
            System.out.println("--------------------------");
        }
    }
}

