//File:         TableSorterDemo.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              3/9/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Table allow user to sort entries according column values
//.             used for displaying bottleneck analysis results

package NetLayout;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;

public class TableSorterDemo extends JFrame {
    private boolean DEBUG = false;
    Net net;

    public TableSorterDemo(Net n) {       
        super("Component Delay Analysis Result");
        net = n;

        MyTableModel myModel = new MyTableModel();
        ComponentDelayList cdl = net.cdl;
        myModel.setSize(cdl.v.size());
        for (int i=0; i< cdl.v.size(); i++) {
            ComponentDelay cd = (ComponentDelay) cdl.v.elementAt(i);
            String name = net.getNameWithID(cd.ID);
            myModel.setValueAt(name, i, 0);
            myModel.setValueAt(new Integer(cd.ID), i, 1);
            myModel.setValueAt(new Double(cd.queueingTime), i, 2);
            myModel.setValueAt(new Double(cd.msgProcessingTime), i, 3);
            myModel.setValueAt(new Double(cd.propagationTime), i, 4);
            myModel.setValueAt(new Double(cd.transmissionTime), i, 5);
            myModel.setValueAt(new Double(cd.retrieveTime), i, 6);
            myModel.setValueAt(new Double(cd.totalDelay), i, 7);
            myModel.setValueAt(new Double(cd.percentage), i, 8);
            myModel.setValueAt(new Integer(cd.rank), i, 9);
        }

        TableSorter sorter = new TableSorter(myModel); //ADDED THIS
        //JTable table = new JTable(myModel);          //OLD
        JTable table = new JTable(sorter);             //NEW
        sorter.addMouseListenerToHeaderInTable(table); //ADDED THIS
        table.setPreferredScrollableViewportSize(new Dimension(500, 70));

        //Create the scroll pane and add the table to it. 
        JScrollPane scrollPane = new JScrollPane(table);

        //Add the scroll pane to this window.
        getContentPane().add(scrollPane, BorderLayout.CENTER);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }

    class MyTableModel extends AbstractTableModel {
        final String[] columnNames = {"Name",
                                "ID",
                                "QueueingDelay",
                                "MsgProcDelay",
                                "PropagationDelay",
                                "TransmissionDelay",
                                "RetrieveDelay",
                                "TotalDelay",
                                "Percentage",
                                "Rank"};                                
        Object[][] data;
        public void setSize(int row) {
          data = new Object[row][columnNames.length];
        }


        public Object getValueAt(int row, int col) {
            return data[row][col];
        }
        public int getColumnCount() {
            return columnNames.length;
        }
        
        public int getRowCount() {
            return data.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }



        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * Don't need to implement this method unless your table's
         * editable.
         */
        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears onscreen.
            if (col < 2) { 
                return false;
            } else {
                return true;
            }
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */
        public void setValueAt(Object value, int row, int col) {
            if (DEBUG) {
                System.out.println("Setting value at " + row + "," + col
                                   + " to " + value
                                   + " (an instance of " 
                                   + value.getClass() + ")");
            }

            data[row][col] = value;
            fireTableCellUpdated(row, col);

            if (DEBUG) {
                System.out.println("New value of data:");
                printDebugData();
            }
        }

        private void printDebugData() {
            int numRows = getRowCount();
            int numCols = getColumnCount();

            for (int i=0; i < numRows; i++) {
                System.out.print("    row " + i + ":");
                for (int j=0; j < numCols; j++) {
                    System.out.print("  " + data[i][j]);
                }
                System.out.println();
            }
            System.out.println("--------------------------");
        }
    }
}

