//File:         ProbingAgent.java
//Project:      ProbeSim
//Version:      0.2
//Designer:     C. Edward Chow
//Revision:
//              1/14/2000  C. Edward Chow
//Bug Report:   send to chow@cs.uccs.edu
//Description:  Simulator for Internet Network Probing Research
//              Implement Jingsha's Time Gap Divergence Method
//              with linear time gap and equal bandwidth time gap
//              multiple iterations in zoom in process for more accurate
//              bandwidth range estimation
//              early out in iteration for fast start of the following
//              iterations
//

package NetLayout;

import java.io.*;
import java.util.*; // for StringTokenizer and Vector
import java.text.*;
import java.awt.geom.Point2D;
import java.awt.Window;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ProbingAgent extends Node {
  double divergeThreshhold = 0.2;
  int maxIteration = 3;
  int msgSize, noOfMsgs;
  double bwLow, bwHigh;
  BWRange curBWR;
  BWRange bwr;
  double intervalStep;
  double interval;    // the time interval btw probing msgs of current round
  int reqID;     // sequence number for the probing msg
  double bwAvailble;  // the estimated results
  double bwPrevious;  // previous round's measurement result
  double errorMargin; // how precise we want it to be
  int probingResponderID;
  int iteration;
  int tableCount = 1;
  double iterationEndTime;
  Random rTime;  // slight deviation of probing round start time for detecting
                 // off period


  double startingTime;  // probing starting time, need to wait for traffic
                        // simulation to reach steady state
  double[] sendingTimeGap;
  double[] receivingTimeGap;
  double[] absSendingTimeGap;          //compared with sendingTime[1]
  double[] absReceivingTimeGap;        //compared with receivingTime[1]
  double[] sendingTime;
  double[] receivingTime;
  boolean roundNotFinished = false;

  public ProbingAgent(Net net, int x, int y) {
    this.net = net;
    ID = net.objID++;
    net.addComponentDelay(ID);
    this.x = x;
    this.y = y;
    type = net.PROBE;
    name =  "Probe" + net.objNo[type]++;
    msgSize = 4096; // in bits
    noOfMsgs = 10;
    rTime = new Random(ID);

    reqID= 1; // initialize
    iteration = 1;
    curBWR = new BWRange();
    bwr = new BWRange();

    bwLow = 10;    // in bits
    bwHigh = 1E7;
    bwPrevious = 10;
    startingTime = 100;
    interval = msgSize/bwLow;
    intervalStep = msgSize*(bwHigh-bwLow)/(bwHigh*bwLow*(noOfMsgs-2));
    errorMargin = intervalStep;   // can depend on convergence of the method
    net.nodes.addElement(this);
  }
  public ProbingAgent(Net net, StringTokenizer st, int type) {
    this.type = type;
    this.net = net;

    AttrValue av = new AttrValue(st);;
    try {
      name = av.value;
      av = new AttrValue(st);
      ID = Integer.parseInt(av.value);
      net.addComponentDelay(ID);
      av = new AttrValue(st);
      x = Integer.parseInt(av.value);
      av = new AttrValue(st);
      y = Integer.parseInt(av.value);
      rTime = new Random(ID);

      reqID= 1; // initialize
      iteration = 1;
      curBWR = new BWRange();
      bwr = new BWRange();

      av = new AttrValue(st);
      probingResponderID = Integer.parseInt(av.value);

      av = new AttrValue(st);
      Double r = Double.valueOf(av.value);
      lon = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      lat = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      power = r.doubleValue();
      av = new AttrValue(st);
      msgSize = Integer.parseInt(av.value);
      av = new AttrValue(st);
      noOfMsgs = Integer.parseInt(av.value);
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      bwLow = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      bwHigh = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      bwPrevious = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      startingTime = r.doubleValue();
      av = new AttrValue(st);
      r = Double.valueOf(av.value);
      errorMargin = r.doubleValue();
      interval = msgSize/bwLow;
      intervalStep = msgSize*(bwHigh-bwLow)/(bwHigh*bwLow*(noOfMsgs-2));

    } catch (NumberFormatException e) {
      System.err.println("Could not convert string to int or double " + av.value);
    }
    localTime = 0.0;
    net.nodes.addElement(this);
    net.objNo[type]++;
    //System.out.println(toString());
  }
  public String toString() {
    return "type="+type+" name="+name+" ID="+ID+
      " x="+x+" y="+y+" "+
      " probingResponderID="+probingResponderID+
      " lon="+lon+" lat="+lat+" power="+power+
      " msgSize="+msgSize+" noOfMsg="+noOfMsgs+
      " bwLow="+bwLow+" bwHigh="+bwHigh+
      " bwPrevious="+bwPrevious+
      " startingTime="+startingTime+
      " errorMargin="+errorMargin+
      linkListToString()+
      "\n";
  }
  public BWRange findBWRange() {
     BWRange bwr = new BWRange();
     double gapDifference;
     double absGapDifference;
     boolean notDiverge = true;
     int i;
 //    for (i=1; i<= sendingTime.length-1 & notDiverge; i++) {
     for (i=2; i<= sendingTime.length & notDiverge; i++) {
        gapDifference = receivingTimeGap[i]-sendingTimeGap[i];
        absGapDifference = absReceivingTimeGap[i]-absSendingTimeGap[i];
 //     if (gapDifference > divergeThreshhold) {
        if (absGapDifference > divergeThreshhold) {    //we want it to be always compared with the roundTripTime of #1 pakcet.
           if (i == 2) bwr.lowerBound = curBWR.lowerBound;
           else bwr.lowerBound = msgSize/sendingTimeGap[i-2];
//         else bwr.lowerBound = msgSize/sendingTimeGap[i-1];
//         bwr.upperBound = msgSize/sendingTimeGap[i];
           bwr.upperBound = msgSize/sendingTimeGap[i-1];
           notDiverge = false;
           return bwr;
        }
     }

     bwr.lowerBound = msgSize/sendingTimeGap[i-1];   // try last bw gap
     bwr.upperBound = 2*curBWR.upperBound;   // should suggest higer value then bwHigh

     return bwr;
  }
  public String ns(double value) {
     DecimalFormat doubleDF = new DecimalFormat("##0.000");
     return ""+doubleDF.format(value);
  }

  public void genIterationResultsTXT() {

     String trafficTypeStr;
     if  (net.trafficType == net.ConstantBitRate)
        trafficTypeStr = "Tcb";
     else if (net.trafficType == net.ConstantClientPacket)
        trafficTypeStr = "Tcp";
     else if (net.trafficType == net.RandomWebTraffic)
        trafficTypeStr = "Trw";
     else trafficTypeStr = "Tss";
     String method;
     if  (net.probingMethod == net.TimeGapDivergeMethod)
        method = "Md";
     else if (net.probingMethod == net.CProbeMethod)
        method = "Mc";
     else method = "Mr";  // round trip difference method
     String maxIterationStr = "I"+maxIteration;
     String msgSizeStr = "S"+msgSize;
     String noOfMsgsStr = "M"+noOfMsgs;
     String probingPeriodStr = "P"+net.probingPeriod;
     String fileName = net.fileName+trafficTypeStr+method+maxIterationStr+
       msgSizeStr+noOfMsgsStr+probingPeriodStr+"R"+net.probingRound+"I"+iteration+".txt";
     System.out.println("...generated fileName="+fileName);
    try {
     PrintWriter txtPW = new PrintWriter( new OutputStreamWriter(
                                 new FileOutputStream("data/"+fileName)));
  /*   PrintWriter txtPW = new PrintWriter( new OutputStreamWriter(System.out));     */
     System.out.println("...writing TXT... "+ fileName);

     txtPW.println("Network="+net.fileName+", Round="+net.probingRound+", Iteration="+iteration+", ProbingPeriod="+net.probingPeriod);
     txtPW.println("ProbingPeriod="+net.probingPeriod+", Network="+net.fileName+", TrafficType="+trafficTypeStr+", Method="+method+", MaxIteration="+maxIteration+", MsgSize="+msgSize+", NoOfMsgs="+noOfMsgs);


     txtPW.println("sendingTime");
  //   System.out.println("Writing sendingTime to files");
     for (int i=1; i<= sendingTime.length-1; i++) {
        txtPW.println(i);
        if (sendingTime[i] <= iterationEndTime)
           txtPW.println(sendingTime[i]*1000000);
        else
           txtPW.println("sending out of date");
      }

      txtPW.println("receivingTime");
      for (int i=1; i<= sendingTime.length-1; i++) {
        txtPW.println(i);
        if (receivingTime[i] <= iterationEndTime)
           txtPW.println( ( receivingTime[i]-(receivingTime[1]-sendingTime[1]) ) * 1000000 );
        else
           txtPW.println("receiving out of date");
      }
      txtPW.close();
    }  catch (IOException e) {
        System.err.println("File not open properly\n"+e.toString());
        System.exit(1);
    } //end of catch
  }  //end of genIteraionResultsTXT

  public void genProbingResultsHTML(BWRange bwr) {

     Link l = (Link) net.trackList.firstElement();
     double bwInkbps = l.bandwidth/1000;
     DecimalFormat doubleDF = new DecimalFormat("##0");
     String bwInkbpsStr = "B"+doubleDF.format(bwInkbps)+"k";
     String trafficTypeStr;
     if  (net.trafficType == net.ConstantBitRate)
        trafficTypeStr = "Tcb";
     else if (net.trafficType == net.ConstantClientPacket)
        trafficTypeStr = "Tcp";
     else if (net.trafficType == net.RandomWebTraffic)
        trafficTypeStr = "Trw";
     else trafficTypeStr = "Tss";
     String method;
     if  (net.probingMethod == net.TimeGapDivergeMethod)
        method = "Md";
     else if (net.probingMethod == net.CProbeMethod)
        method = "Mc";
     else method = "Mr";  // round trip difference method
     String maxIterationStr = "I"+maxIteration;
     String msgSizeStr = "S"+msgSize;
     String noOfMsgsStr = "M"+noOfMsgs;
     String probingPeriodStr = "P"+net.probingPeriod;
     String fileName = net.fileName+bwInkbpsStr+trafficTypeStr+method+maxIterationStr+
       msgSizeStr+noOfMsgsStr+probingPeriodStr+"R"+net.probingRound+"I"+iteration+".htm";
     System.out.println("...generated fileName="+fileName);
   try {
       PrintWriter htmPW = new PrintWriter( new OutputStreamWriter(
                                 new FileOutputStream("data/"+fileName)));
      /* PrintWriter htmPW = new PrintWriter( new OutputStreamWriter(System.out);    */
     System.out.println("...writing... "+ fileName);
     htmPW.println("<html><title>Probing Result for "+fileName+"</title><body>");
     htmPW.println("<center>Round="+net.probingRound+", Iteration="+iteration+" Probing Bandwidth Result: BW within "+bwr.toString());
     htmPW.println("<table BORDER=3 CELLSPACING=3 CELLPADDING=3 BGCOLOR=\"#C0C0C0\"><tr><th>Seq#</th><th>SendingTime</th>");
     htmPW.println("           <th>ReceivingTime</th><th>RTTime</th>");
     htmPW.println("           <th>BW Demand</th><th>STGap</th>");
     htmPW.println("           <th>RTGap</th><th>Jitter</th></tr>");

     boolean display = true;

     for (int i=1; i<= sendingTime.length-1; i++) {
        htmPW.println("       <tr><td>"+i+"</td>");
        if (sendingTime[i] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns(sendingTime[i]*1000)+"</td>");
        else
           htmPW.println("           <td align=right>&nbsp;</td>");

        if (display && receivingTime[i] <= iterationEndTime) {
           htmPW.println("           <td align=right>"+ns(receivingTime[i]*1000)+"</td>");

           if (receivingTime[i] == iterationEndTime) {
              display = false;
           }
        } else
           htmPW.println("           <td align=right>&nbsp;</td>");

        if (display && receivingTime[i] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns((receivingTime[i]-sendingTime[i])*1000)+"</td>");
        else if (receivingTime[i] == iterationEndTime) {
           htmPW.println("           <td align=right>"+ns((receivingTime[i]-sendingTime[i])*1000)+"</td>");
        } else
           htmPW.println("           <td align=right>&nbsp;</td>");
        if (sendingTime[i] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns(msgSize/sendingTimeGap[i])+"</td>");
        else
           htmPW.println("           <td align=right>&nbsp;</td>");
        if (sendingTime[i] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns(absSendingTimeGap[i]*1000)+"</td>");
        else
           htmPW.println("           <td align=right>&nbsp;</td>");
        if (display && i< sendingTime.length-1 && receivingTime[i+1] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns(absReceivingTimeGap[i]*1000)+"</td>");
        else
           htmPW.println("           <td align=right>&nbsp;</td>");
        if (display && i< sendingTime.length-1 && receivingTime[i+1] <= iterationEndTime)
           htmPW.println("           <td align=right>"+ns((absReceivingTimeGap[i]-absSendingTimeGap[i])*1000)+"</td>");
            else
           htmPW.println("           <td align=right>&nbsp;</td>");
        }
     htmPW.println("</table></center></body></html>");
     htmPW.close();
  } catch (IOException e) {
        System.err.println("File not open properly\n"+e.toString());
        System.exit(1);
    }

  }
  public void setupNextInteration(BWRange bwr) {
      curBWR.lowerBound = bwr.lowerBound;
      curBWR.upperBound = bwr.upperBound;

      for (int i=1; i<=noOfMsgs; i++) {
         receivingTime[i]= 0.0; // used for detecting msg loss
      }
      // prepare time gap array data
      //sendingTimeGap = new double[noOfMsgs+1]; // waste one for 1-based index
      double startProbeTime = net.startProbeTime+net.probingPeriod*net.probingRound;
      if (startProbeTime < localTime) startProbeTime = localTime+0.000001;

      startProbeTime = localTime+0.000001;

      sendingTime[1]=startProbeTime;

      EventRecord er = new EventRecord(startProbeTime, ID, ID, ID);
      er.addField("msgType", "probing-request");
      er.addField("iteration", ""+iteration); // need iteration no. for early out stop sending probing msg
      net.println("put probing-request in queue wake me up after "+startProbeTime+"sec.");
      net.el.queueEvent(er);
      net.println("iteration="+iteration+", localTime="+localTime+", startProbeTime="+startProbeTime);
      reqID = 1;  // set first reqID
      for (int i=1; i<noOfMsgs; i++) {
         if (net.equalBWTimeGap) {
           sendingTimeGap[i]= msgSize/(bwr.lowerBound+i*(bwr.upperBound-bwr.lowerBound)/(noOfMsgs-1));
           sendingTime[i+1] = sendingTime[i]+sendingTimeGap[i];
         } else {
           double intervalStepA = msgSize*(bwr.upperBound-bwr.lowerBound)/(bwr.upperBound*bwr.lowerBound*(noOfMsgs-2));
           sendingTimeGap[i]= msgSize/bwr.lowerBound-intervalStepA*(i-1);
           sendingTime[i+1] = sendingTime[i]+sendingTimeGap[i];
         }
      }
      for (int i=1; i<=noOfMsgs; i++) {
         absSendingTimeGap[i] = sendingTime[i]-sendingTime[1];
      }
      //receivingTimeGap = new double[noOfMsgs+1];
      for (int i=1; i<noOfMsgs; i++) {
         receivingTimeGap[i]= 0.0;
      }
      for (int i=1; i<=noOfMsgs; i++) {
         absReceivingTimeGap[i]= 0.0;
      }
  }
  public void eventHandle(EventRecord er) {

    double queueingTime = localTime - er.time;
      if (localTime < er.time) {
        localTime = er.time;
        queueingTime = 0.0;
      }
    String msgType = er.getField("msgType");
    if (msgType.compareTo("init") == 0) {
      if (roundNotFinished) {
        er.time += net.probingPeriod;  // skip to next probing period
        net.el.queueEvent(er);
        net.println("current round not finished! skip to next  probing perid!");
      } else {
      System.out.println("+++ noOfMsg="+noOfMsgs+", bwLow="+bwLow+", bwHigh="+bwHigh);
      curBWR.lowerBound = bwLow;
      curBWR.upperBound = bwHigh;
      iteration = 1;
      // prepare sending time and receiving time array data
      sendingTime = new double[noOfMsgs+1];  // waste one for 1-based index
      receivingTime = new double[noOfMsgs+1];
      for (int i=1; i<=noOfMsgs; i++) {
         receivingTime[i]= 0.0; // used for detecting msg loss
      }
      // prepare time gap array data
      sendingTimeGap = new double[noOfMsgs+1]; // waste one for 1-based index
      absSendingTimeGap = new double[noOfMsgs+1];
      double startProbeTime = net.startProbeTime+net.probingPeriod*net.probingRound;
      if (startProbeTime < localTime) startProbeTime = localTime+0.000001;
      // in case it got delayed
      sendingTime[1]=startProbeTime;
      // er = new EventRecord(startingTime, ID, ID, ID); overwritten by net.startPlotTime
      // compute next round's start probing time with random deviation

      er.time = startProbeTime+net.probingPeriod+rTime.nextDouble()*net.randomStartTimeDeviation;

      net.probingRound++;
      net.el.queueEvent(er);
      net.println("put next round of probing requests in queue, wake me up after "+er.time+"sec.");
      er = new EventRecord(startProbeTime, ID, ID, ID);
      er.addField("msgType", "probing-request");
      er.addField("iteration", ""+iteration);  // need iteration no. for early out stop sending probing msg
      net.println("put probing-request in queue wake me up after "+startProbeTime+"sec.");
      net.el.queueEvent(er);
      net.println("localTime="+localTime+", startProbeTime="+startProbeTime);
      reqID = 1;  // set first reqID
      for (int i=1; i<noOfMsgs; i++) {
         if (net.equalBWTimeGap) {
           sendingTimeGap[i]= msgSize/(bwLow+i*(bwHigh-bwLow)/(noOfMsgs-1));
           sendingTime[i+1] = sendingTime[i]+sendingTimeGap[i];

           net.println("i="+i+", sendingTimeGap[i]="+sendingTimeGap[i]+",sendingTime[i+1]="+sendingTime[i+1]);
         } else {
           sendingTimeGap[i]= msgSize/bwLow-intervalStep*(i-1);
           sendingTime[i+1] = sendingTime[i]+sendingTimeGap[i];
           net.println("i="+i+", sendingTimeGap[i]="+sendingTimeGap[i]+",sendingTime[i+1]="+sendingTime[i+1]);
         }
      }

      for (int i=1; i<=noOfMsgs; i++) {
         absSendingTimeGap[i] = sendingTime[i] - sendingTime[1];
      }

      receivingTimeGap = new double[noOfMsgs+1];
      absReceivingTimeGap = new double[noOfMsgs+1];

      for (int i=1; i<noOfMsgs; i++) {
         receivingTimeGap[i]= 0.0;

      }
      for (int i=1; i<=noOfMsgs; i++)  {
         absReceivingTimeGap[i]= 0.0;
      }

      System.out.println("recieved init event, generate probing request\n");
     }
    }
    if (msgType.compareTo("probing-request") == 0) {
      int rcvIteration = Integer.parseInt(er.getField("iteration"));
      if (rcvIteration < iteration) return;  // request of early iteration
      net.println("recieved probing-request event, generate probing request\n");


      Link l = getFirstLink();  // find the cable connected to the ProbingAgent
      if (l == null) {
        System.out.println("This ProbingAgent is not connected to the network.");
        return;
      }

      // calculate the transmission time


      double transmissionTime = msgSize/l.bandwidth;

      double propagationTime = l.distance/net.signalPropagationSpeed;

      double nextChannelAvailableTime = l.getChannelAvailableTime(this);
      double timeDifference = nextChannelAvailableTime - localTime;
      double time = 0.0;
      if (timeDifference > 0) {
         time = nextChannelAvailableTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, nextChannelAvailableTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, nextChannelAvailableTime);
      } else {
         time = localTime+transmissionTime+propagationTime;
         l.setChannelAvailableTime(this, localTime+transmissionTime);
         l.sendMsgToChannel(this, msgSize, localTime);
      }

      sendingTime[reqID] = localTime;
      er.addField("startTime", ""+localTime);
      // save er.time since we are resuing the old init record as probing-request er
      er.time = time;
      er.next = l.n2.ID;
      er.src = ID;
      er.dst = probingResponderID;
      er.addField("msgType", "probing-request");
      er.addField("msgSize", ""+msgSize);
      er.addField("reqID", ""+reqID);
      er.addField("iteration", ""+iteration);
      er.addField("round", ""+net.probingRound);

      er.addField("route-qt", this.name+":0.0, ");

      double accQueueingTime = 0.0;
      er.addField("queueingTime", ""+accQueueingTime);

      double accMsgProcessingTime = (net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      er.addField("msgProcessingTime", ""+accMsgProcessingTime);

      double accTransmissionTime = transmissionTime;
      er.addField("transmissionTime", ""+accTransmissionTime);

      double accPropagationTime = propagationTime;
      er.addField("propagationTime", ""+accPropagationTime);

      String s = ""+ID+":MPT="+accMsgProcessingTime+" "+
                 l.ID+":PT="+propagationTime+",TT="+transmissionTime;
      er.addField("componentDelay", s);


      // generate animation event if the animation flag is set on
      int p2x, p2y;
      if (net.animationOn) {
        if (l.n2.type == Net.MALINK) {
          p2x = l.x; p2y = l.y;
        } else {
          p2x = l.n2.x+16; p2y = l.n2.y+16;
        }
        net.animate(er, localTime, propagationTime, x+16, y+16, p2x, p2y);
      }

      net.el.queueEvent(er);
      net.println("*** time="+localTime+" name="+name+
                  " send msg with round="+net.probingRound+
                  " iteration="+iteration+
                  " reqID="+reqID+
                  " msgSize="+msgSize);
      reqID++;
      if (reqID <= noOfMsgs) {
         er = new EventRecord(sendingTime[reqID], ID, ID, ID);
         er.addField("msgType", "probing-request");
         er.addField("iteration", ""+iteration); // need iteration no. for early out stop sending probing msg
         net.println("put next probing-request in queue, wake me up after "+sendingTime[reqID]+"sec.");
         net.el.queueEvent(er);
      }
    }
    if (msgType.compareTo("probing-response") == 0) {

      totalQueueingTime += queueingTime;
      noOfRequests++;
     try {
      //Double d = Double.valueOf(er.getField("retrieveTime"));
      //double retrieveTime = d.doubleValue();

      Double d = Double.valueOf(er.getField("queueingTime"));
      double accQueueingTime = d.doubleValue();
      accQueueingTime += queueingTime;

      Double r = Double.valueOf(er.getField("msgSize"));
      double msgSize = r.doubleValue();
      double msgProcessingTime = (net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power;
      d = Double.valueOf(er.getField("msgProcessingTime"));
      double accMsgProcessingTime = d.doubleValue();
      accMsgProcessingTime += msgProcessingTime;

      d = Double.valueOf(er.getField("transmissionTime"));
      double accTransmissionTime = d.doubleValue();

      d = Double.valueOf(er.getField("propagationTime"));
      double accPropagationTime = d.doubleValue();

      int rcvreqID = Integer.parseInt(er.getField("reqID"));
      int rcvIteration = Integer.parseInt(er.getField("iteration"));
      int rcvRound = Integer.parseInt(er.getField("round"));


      localTime += (net.msgProcessingTime+msgSize/net.msgProcessingSpeed)/power+
                    queueingTime;
      receivingTime[rcvreqID] = localTime;

      net.println("*** time="+localTime+" name="+name+
                  " receive msg with round="+rcvRound+
                  " iteration="+rcvIteration+
                  " reqID="+rcvreqID+
                  " msgSize="+msgSize);

      if (rcvIteration < iteration) {
         net.println("!!! round="+net.probingRound+" rcvIteration="+rcvIteration+" < iteration");
         return;    // discard this msg, another iteration already starts
      }

      absReceivingTimeGap[rcvreqID] = receivingTime[rcvreqID] - receivingTime[1];
/*
      if (rcvreqID == 10)  {
        System.out.println("receivingTime of the last paket is"+receivingTime[rcvreqID]+", receivingTime of the first pkt is "+receivingTime[1]+", absRcvTG="+absReceivingTimeGap[rcvreqID]);
      }
*/
      if (rcvreqID != 1 && receivingTime[rcvreqID-1] != 0.0) {
        receivingTimeGap[rcvreqID-1] = receivingTime[rcvreqID] - receivingTime[rcvreqID-1];

        //  early stop,  multiple iterations
        double gapDifference = receivingTimeGap[rcvreqID-1]-sendingTimeGap[rcvreqID-1];
        double absGapDifference = absReceivingTimeGap[rcvreqID]-absSendingTimeGap[rcvreqID];
        System.out.println("!!! round="+net.probingRound+", iteration="+iteration+
                          ", receive msg with reqID="+rcvreqID+", gapDifference="+gapDifference+
                          ", absGapDifference="+absGapDifference);

/*Modified(voided) by Jihui, Jihui1_1
      if (gapDifference > divergeThreshhold) {
           // report average of the current range of probing results
           if (rcvreqID == 2) bwr.lowerBound = curBWR.lowerBound;
           else bwr.lowerBound = msgSize/sendingTimeGap[rcvreqID-2];
           bwr.upperBound = msgSize/sendingTimeGap[rcvreqID-1];
           System.out.println("!!early out  time="+localTime+" iteration="+iteration+" plot probing result, bwr="+bwr.average());
           net.plotProbingResults.c1.updateProbingData(localTime, bwr.average());
           System.out.println("!!!end of round="+net.probingRound+", iteration="+iteration+
                       ", bwlow="+bwr.lowerBound+", bwHigh="+bwr.upperBound);
           iterationEndTime = localTime;
           genProbingResultsHTML(bwr);
           if (tableCount <= net.NoOfTableProbingResults)  {
             net.tableProbingResults[net.probingRound*maxIteration+iteration-1] = new TableProbingResult(this, bwr);
             tableCount++;
           }
           iteration++;
           if (iteration <= maxIteration) {
              // start next iteration
              setupNextInteration(bwr);
           }
           return;
        }
End of  Modified by Jihui, Jihui1_1*/

      }

      // need to add time out event to simulate the lost of last probing response

      if (rcvreqID == noOfMsgs) {
        BWRange bwr;
        bwr = findBWRange();
        System.out.println("!!plotProbingResult rcvID=noOfmsgs  time="+localTime+" iteration="+iteration+" plot probing result, bwr="+bwr.average());
        net.plotProbingResults.c1.updateProbingData(localTime, bwr.average());
        System.out.println("!!! rcv all msg, round="+net.probingRound+", iteration="+iteration+
                       ", bwlow="+bwr.lowerBound+", bwHigh="+bwr.upperBound);
        iterationEndTime = localTime;
        genProbingResultsHTML(bwr);

  //      PlotEachIteration plotEachIteration = new PlotEachIteration(this);
  //      plotEachIteration.setSize(680, 480);
        PlotEachIteration plotEachIteration = new PlotEachIteration(this);
        plotEachIteration.show();

        genIterationResultsTXT();

  //     plotEachIteration.pip.drawing();

        if (tableCount <= net.NoOfTableProbingResults)  {
           net.tableProbingResults[net.probingRound*maxIteration+iteration-1] = new TableProbingResult(this, bwr);
           /*added by Jihui, Jihui1_5*/

        //   Point location = net.plotEachIteration[net.probingRound*maxIteration+iteration-1].getLocation();

        //   net.plotEachIteration[net.probingRound*maxIteration+iteration-1] = new PlotEachIteration(this, location);



           /*End of add by Jihui, Jihui_5*/
           tableCount++;
        }
        iteration++;
        if (iteration <= maxIteration) {
              // start next iteration
              setupNextInteration(bwr);
        }
      }




      /*
      d = Double.valueOf(er.getField("startTime"));
      double startTime = d.doubleValue();
      double endToEndRespTime = localTime - startTime;


      net.println("*** endToEndResptime="+endToEndRespTime);
      net.println("*** accQueueingTime="+accQueueingTime+
            ", percentage="+accQueueingTime*100.0/endToEndRespTime);
      net.println("*** accMsgProcessingTime="+accMsgProcessingTime+
            ", percentage="+accMsgProcessingTime*100.0/endToEndRespTime);
      net.println("*** accTransmissionTime="+accTransmissionTime+
            ", percentage="+accTransmissionTime*100.0/endToEndRespTime);
      net.println("*** accPropagationTime="+accPropagationTime+
            ", percentage="+accPropagationTime*100.0/endToEndRespTime);
      //net.println("*** retrieveTime="+retrieveTime+
      //      ", percentage="+retrieveTime*100.0/endToEndRespTime);

      net.println("*** route-qt="+er.getField("route-qt")+", "+name+":"+queueingTime);

      // Chow: not part of the regular traffic skip the following calculation
      //net.totalEndToEndRespTime += endToEndRespTime;
      //net.noOfFinishedRequests++;

      // componentDelayAnalysis
      String cds = er.getField("componentDelay");
      String ProbingAgentCDS = " "+ID+":QT="+queueingTime+",MPT="+msgProcessingTime;
      ComponentDelayList cdl = new ComponentDelayList(net, ""+net.noOfFinishedRequests, cds+ProbingAgentCDS);
      // net.logPrintln("local seesion CDA:\n"+cdl.toString());
      net.addComponentDelayList(cdl);
      net.logPrintln("===Request "+net.noOfFinishedRequests+": "+cds+ProbingAgentCDS);
      if (net.noOfFinishedRequests%100==1) {
        net.logPrintln("local session CDA:\n"+cdl.toString());
        net.cdl.sort();
        net.logPrintln("Global List CDA:\n"+net.cdl.toString());
      }
      */
     } catch (NumberFormatException e) {
      System.err.println("ProbingAgent.java: Could not convert string to double " + e.toString());
      net.logPW.close();
      System.exit(1);
     }

    }
  }

}
